use enum_index_derive::{EnumIndex,IndexEnum};
use strum_macros::EnumString;
use crate::message_species;
#[derive(Debug,Clone,EnumIndex, IndexEnum, EnumString)]
pub enum Family{
    FmARCH,                     //0,   This is used by the WACP architecture to provide messages for standard features.
    FmECG,                      //1,   This is used for the ElctroCardioGram (ECG) components.
    FmIP,                       //2,   This is used for the Impedance Pneumography (IP) Respiration components
    FmTEMP,                     //3,   This family is used for all components that perform temperature measurements (SureTemp and Oral/Skin/Core). The different types will be differentiated by species.
    FmSPO2,                     //4,   This is used for the SpO2 component that interfaces with all forms of OEM SpO2 boards.
    FmNIBP,                     //5,   This is used for all forms of Non-Invasive Blood Pressure (NIBP).
    FmICG,                      //6,   Non-Invasive Cardiac Output (Impedance CardioGraphy, ICG)
    FmHEMODYNAMICS,             //7,   Hemodynamic Values
    FmCO2,                      //8,   This is used for mainstream and sidestream CO2 monitoring components.
    FmIBP,                      //9,   This is used for Invasive Blood Pressure components (IBP).
    FmCARDIACOUTPUT,            //10,  This is used for all forms of cardiac output components.
    FmUI,                       //11,  This is used for all manner of user interface components.
    FmBARCODE,                  //12,  This is used for components that interface with a barcode reader.
    FmCTR,                      //13,  These are the main processing node controllers. An example is a “Monet” controller.
    FmPATIENT,                  //14,  Used for context for test results.
    FmUSER,                     //15,  Used for context in test results.
    FmGLUCOSE,                  //16,  This is used for Glucose components.
    FmBODYMASSINDEX,            //17,  This is used for body mass index components.
    FmRESPIRATION,              //18,  This is used for respiration components.
    FmWEIGHT,                   //19,  This is used for weight components.
    FmINSURANCE,                //20,  This is used for insurance definition.
    FmPHYSICIAN,                //21,  This is used for physician definition.
    FmSITE,                     //22,  This is used for site definition.
    FmSESSION,                  //23,  This is used for session components.
    FmDEVICE,                   //24,  Details the basic common attributes of a device or instrument. This may include model, serial etc…
    FmINTERPRETATION,           //25,  This is used for interpretation components generated by a ECG analysis.
    FmHEIGHT,                   //26,  This is used for height component definition.
    FmSPIRO,                    //27,  This used for Spirometery components.
    FmTRAP,                     //28,  An architecture level definition for error and exceptions generated by application processes and OSI framework members. Used in network communications.
    FmRENDEZVOUS,               //29,  A connection based component with messages that facilitate network connection sequences on multiple media.
    FmCONNECTION,               //30,  A connection management component with base capabilities for managing connections (wireless) that are not guaranteed.
    FmWAVESAMPLE,               //31,  The representation of points in any type of measured recording over time.
    FmWAVE,                     //32,  The representation of points in any type of measured recording over time.
    FmDEV01,                    //33,  Reserved for development use only.
    FmDEV02,                    //34,  ''
    FmDEV03,                    //35,  ''
    FmDEV04,                    //36,  ''
    FmDEV05,                    //37,  ''
    FmDEV06,                    //38,  ''
    FmDEV07,                    //39,  ''
    FmDEV08,                    //40,  ''
    FmDEV09,                    //41,  ''
    FmDEV10,                    //42,  ''
    FmDEV11,                    //43,  ''
    FmDEV12,                    //44,  ''
    FmDEV13,                    //45,  ''
    FmDEV14,                    //46,  ''
    FmDEV15,                    //47,  ''
    FmDEV16,                    //48,  ''
    FmDEV17,                    //49,  ''
    FmDEV18,                    //50,  ''
    FmDEV19,                    //51,  ''
    FmDEV20,                    //52,  ''
    FmDEV21,                    //53,  ''
    FmDEV22,                    //54,  ''
    FmDEV23,                    //55,  ''
    FmDEV24,                    //56,  ''
    FmDEV25,                    //57,  ''
    FmDEV26,                    //58,  ''
    FmDEV27,                    //59,  ''
    FmDEV28,                    //60,  ''
    FmDEV29,                    //61,  ''
    FmDEV30,                    //62,  ''
    FmDEV31,                    //63,  ''
    FmDEV32,                    //64,  ''
    FmDEV33,                    //65,  ''
    FmDEV34,                    //66,  ''
    FmDEV35,                    //67,  ''
    FmDEV36,                    //68,  ''
    FmDEV37,                    //69,  ''
    FmDEV38,                    //70,  ''
    FmDEV39,                    //71,  ''
    FmDEV40,                    //72,  ''
    FmDEV41,                    //73,  ''
    FmDEV42,                    //74,  ''
    FmDEV43,                    //75,  ''
    FmDEV44,                    //76,  ''
    FmDEV45,                    //77,  ''
    FmDEV46,                    //78,  ''
    FmDEV47,                    //79,  ''
    FmDEV48,                    //80,  ''
    FmDEV49,                    //81,  ''
    FmDEV50,                    //82,  ''
    FmGLOBALMEASUREMENTS,       //83,  
    FmLEADIMPEDANCEARRAY,       //84,  
    FmLEADIMPEDANCE,            //85,  
    FmMEASUREMENTMATRIX,        //86,  
    FmLEADMEASUREMENT,          //87,  
    FmQRSANNOTARRAY,            //88,  
    FmQRSCOMPLEX,               //89,  
    FmPACERARRAY,               //90,  
    FmPACERINFO,                //91,  
    FmDTCOLLECTION,             //92,  Collection Object
    FmWACP,                     //93,  WACP Object
    FmFLASH,                    //94,  Flash Programming Object
    FmVERSION,                  //95,  
    FmPRINTER,                  //96,  
    FmRECORD,                   //97,  Results generated by a device.
    FmTCPIP,                    //98,  TCPIP Settings
    FmLOCATIONID,               //99,  Network Identification
    FmPAIN,                     //100, 
    FmCLINICIAN,                //101, 
    FmCALIBRATION,              //102, 
    FmPATIENTLIST,              //103, 
    FmFAX,                      //104, 
    FmMODULE,                   //105, 
    FmMODEM,                    //106, 
    FmNETWORK,                  //107, 
    FmTEST,                     //108, 
    FmENCRYPTION,               //109, 
    FmTOC,                      //110, 
    FmBATTERY,                  //111, 
    FmFILESYS,                  //112, 
    FmMOIBSTRING,               //113, 
    FmERROR,                    //114, 
    FmCOMMAND_LINE,             //115, 
    FmDICTIONARY,               //116, 
    FmNUMERIC,                  //117, 
    FmCHANNEL,                  //118, 
    FmORDER,                    //119, 
    FmQUERY,                    //120, 
    FmTYMP,                     //121, 
    FmDATATRANSFER,             //122, 
    FmLIST,                     //123,
    FmHR,                       //124,
    FmSPHB,                     //125,
    FmHIM,                      //126,
    FmSPOC,                     //127,
    FmSPCO,                     //128,
    FmSPMET,                    //129,
    FmPATIENT_MOTION,           //130,
    FmCUSTOM,                   //131,
    FmMAX                       //132, The FmMAX is a sliding value that will be one enumerated value greater than the last family defined. The actual enumerated value may vary if additional families are added in the Constants Table XML document. This value is used to terminate the family enumeration list.
}

#[derive(Debug,Clone,EnumIndex, IndexEnum, EnumString)]
pub enum Genus{
    GnDATA,         //0,  An aggregate type that represents a number of attributes related to a particular family.
    GnCOMMAND,      //1,  Used to send commands to components.
    GnEVENT,        //2,  Notify the rest of the system that something has taken place.
    GnERROR,        //3,  Used to notify the system that something has occurred that should not have occurred.
    GnPARAMETER,    //4,  Used to send parameter information.
    GnSTATUS,       //5,  The persistent status of the component during the current power up state of the processing node.
    GnNUMERIC,      //6,  Used primarily for patient vital-signs numeric, but is not limited to representation of other numeric data.
    GnRESERVE_A,    //7,  Reserved
    GnCONFIG,       //8,  Used to contain the configuration data of the components.
    GnRESERVE_B,    //9,  Reserved
    GnRESERVE_C,    //10, Reserved
    GnREQUEST,      //11, Used to request that one component send a message directly to another component.
    GnRESERVE_D,    //12, Reserved
    GnRESERVE_E,    //13, Reserved
    GnRESERVE_F,    //14, Reserved
    GnRESPONSE,     //15, Response
    GnRESERVE_G,    //16, Reserved
    GnSTREAM,       //17, Used for messaging while not requiring an ACK or NAK.
    GnDATAEXT,      //18, An aggregate type that represents a number of attributes related to a particular family, extended to 4 bytes.
    GnANNOTATION,   //19, Used to contain data that provides additional information for another object.
    GnMAX           //20, Used to contain data that provides additional information for another object.
}

pub struct Packet{
    pub(crate) packet_length: u32,
    pub(crate) msg:&'static dyn message_species::Message,
    pub(crate) port:Port,
    pub(crate) crc:u16
}

impl Packet{
    pub fn new() -> Self { todo!() }
    pub fn get_length(&self) -> u32 { self.packet_length }
    pub fn get_msg(&self) -> Box<&'static dyn message_species::Message> { Box::new(self.msg) }
    pub fn get_port(&self) -> Port { self.port }
    pub fn get_crc(&self) -> u16 { self.crc }
    fn calculate_size(&mut self) { todo!() }
    fn generate_crc(&mut self) { todo!() }
}

#[derive(Clone,Copy)]
pub enum Port{
    RENDEZVOUS,
    WACP
}

