use super::MessageSpecies;

//Messages relating to the Non-Invasive Blood Pressure system
pub enum NIBPMessage{
    GetBP,              //Request   0x00;
    GetConfig,          //Request   0x01;
    PutBP,              //Response  0x00; requires NIBPData obj
    PutConfig,          //Response  0x01;
    StartNIBP,          //Command   0x00; requires NIBPParam obj
    StopNIBP,           //Command   0x01;
    WriteConfig,        //Command   0x02;
    ReportBP,           //Stream    0x00;
    ZeroNIBP,           //Command   0x03;
    MsgPushEnable,      //Command   0x04; requires NIBPSelect obj
    StreamBP,           //Stream    0x01; requires NIBPData obj
    GetStatus,          //Request   0x02; requires NIBPSelect obj
    PutStatus,          //Response  0x02; requires NIBPStatus obj
    ReportStatus,       //Stream    0x02; requires NIBPStatus obj [only reports primary]
    ClearSafetyStatus,  //Command   0x05; no obj
    ClearPOSTStatus,    //Command   0x06; no obj
    ResetDevice,        //Command   0x07; no obj
    GetCalib,           //Request   0x03; requires NIBPSelect obj
    PutCalib,           //Response  0x03; requires NIBPCal obj
    SetCalib,           //Command   0x08; requires NIBPCal obj
    SaveCalib,          //Command   0x09; requires NIBPSelect obj
    LoadCalib,          //Command   0x0a; requires NIBPSelect obj
    RestoreCalib,       //Command   0x0b; requires NIBPSelect obj
    GetBPSettings,      //Request   0x04;
    PutBPSettings,      //Response  0x04; requires NIBPConfigData obj
    SetBPSettings,      //Command   0x0c; requires NIBPConfigData obj
    SaveBPSettings,     //Comand    0x0d; no obj
    LoadBPSettings,     //Command   0x0e; no obj
    RestoreBPSettings,  //Command   0x0f; no obj
    GetChunk,           //Request   0x05; requires NIBPChunkSelect obj
    PutChunk,           //Response  0x05; requires NIBPChunk obj
    GetChunkSize,       //Request   0x06; requires NIBPSelect obj
    PutChunkSize,       //Response  0x06; requires NIBPSelect obj
    GetMaxChunkID,      //Request   0x07; no obj
    PutMaxChunkID,      //Response  0x07; requires NIBPSelect obj
    GetHWState,         //Request   0x08; requires NIBPSelect obj
    PutHWState,         //Response  0x08; Requires NIBPHWState obj
    SetHWState,         //Command   0x10; Requires NIBPHWState obj
    PulseStepValve,     //Command   0x11; Requires NIBPSelect obj to specify time in units 0.0001 sec
    GetTestLock,        //Request   0x09; Requires NIBPSelect obj
    PutTestLock,        //Response  0x09; Requires NIBPLock
    SetTestLock,        //Command   0x12; requires NIBPLock
    GetRawData,         //Request   0x0a;
    PutRawData,         //Response  0x0a; Requires NIBPRawData obj
    GetPressureData,    //Request   0x0b;
    PutPressureData,    //Response  0x0b; Requires NIBPPressureData obj
}

impl MessageSpecies for NIBPMessage{}
