use super::MessageSpecies;

impl MessageSpecies for TympMessage{}
pub enum TympMessage{
    SetCalInfo,             //Command   0x00, requires CalInfo obj
    GetCalInfo,             //Request   0x00, no obj
    PutCalInfo,             //Response  0x00, requires CalInfo obj
    SetBarcode,             //Command   0x01, requires Barcode obj
    GetBarcode,             //Request   0x01, no obj
    PutBarcode,             //Response  0x01, requires Barcode obj
    GetMemCheck,            //Request   0x02, requires TympMsgArg. IOParamA = requested img
    PutMemCheck,            //Response  0x02, requires TympMsgArg. IOParamA = img and IOParamB = result
    GetMemTest,             //Request   0x03, requires TympMsgArg. IOParamA = memory to test
    PutMemTest,             //Response  0x03, requires TympMsgArg. IOParamA = img and IOParamB = result
    ClearErrorLog,          //Command   0x02, no obj
    RunTymp,                //Command   0x03, requires TympMsgArg. IOParamA = freq, 
                            //                  IOParamB = Ear, IOParamC = reserved
    GetTympData,            //Request   0x04, requires TympMsgArg. IOParamA = freq(reserved), 
                            //                  IOParamB = Ear, IOParamC = reserved
    PutTympData,            //Response  0x04, requires Tymp obj
    GetFineSpeakerGain,     //Request   0x05, no obj
    PutFineSpeakerGain,     //Response  0x05, requires TympMsgArg. Species = Speaker, IOParamA = gain, 
    SetFineSpeakerGain,     //Command   0x04, requires TympMsgArg. Species = Speaker, IOParamA = gain
    GetDefaultSpeakerVolume,//Request   0x06, requires TympMsgArg. Species = Speaker, IOParamA = freq, 
                            //                  IOParamB = Index, IOParamC = Reserved
    PutDefaultSpeakerVolume,//Response  0x06, requires TympMsgArg. Species = Speaker, IOParamA = freq, 
                            //                  IOParamB = Index, IOParamC = Reserved
    SetDefaultSpeakerVolume,//Command   0x05, requires TympMsgArg. Species = Speaker, IOParamA = freq, 
                            //                  IOParamB = Index, IOParamC = Reserved
    GetFineMicGain,         //Request   0x07, requires TympMsgArg. Species = Mic, IOParamA = freq
    PutFineMicGain,         //Response  0x07, requires TympMsgArg. Species = Mic, IOParamA = freq, 
                            //                  IOParamB = gain, IOParamC = Reserved
    SetFineMicGain,         //Command   0x06, requires TympMsgArg. Species = Mic, IOParamA = freq,
                            //                  IOParamB = gain, IOParamC = N/A
    GetCoarseGain,          //Request   0x08, requires TympMsgArg. Species = Speaker | Mic. 
                            //                  IOParamA = freq
    PutCoarseGain,          //Response  0x08, requires TympMsgArg. Species = Speaker | Mic. 
                            //                  IOParamA = freq, IOParamB = Gain
    SetCoarseGain,          //Command   0x07, requires TympMsgArg. Species = Speaker | Mic. 
                            //                  IOParamA = freq, IOParamB = Gain
    GetMicPeakLevel,        //Request   0x09, no obj
    PutMicPeakLevel,        //Response  0x09, requires TympMsgArg. Species = Mic. IOParamA = Peak
    SetSpeakerState,        //Command   0x08, requires TympMsgArg. Species = Speaker. IOParamA = State
    StartTone,              //Command   0x09, requires TympMsgArg. Species = Tone. IOParamA = freq
    GetCurrentAdmittance,   //Request   0x0a, no obj
    PutCurrentAdmittance,   //Response  0x0a, requires TympMsgArg. Species = Admittance 
                            //                  IOParamA = Admittance
    GetPressureGain,        //Request   0x0b, no obj
    PutPressureGain,        //Response  0x0b, requires TympMsgArg. Species = Pressure
                            //                  IOParamA = Gain
    SetPressureGain,        //Command   0x0a, requires TympMsgArg. Species = Pressure
                            //                  IOParamA = Gain
    GetPressureOffset,      //Request   0x0c, no obj
    PutPressureOffset,      //Response  0x0c, requires TympMsgArg. Species = Pressure
                            //                  IOParamA = Offset
    SetPressureOffset,      //Command   0x0b, requires TympMsgArg. Species = Pressure
                            //                  IOParamA = Offset
    AutoZero,               //Command   0x0c, no obj
    GetPressure,            //Request   0x0d, no obj
    PutPressure,            //Response  0x0d, requires TympMsgArg. Species = Pressure
                            //                  IOParamA = Pressure
    SetValve,               //Command   0x0d, requires TympMsgArg. Species = Pressure
                            //                  IOParamA = State
    PressureSweep,          //Command   0x0e, requires TympMsgArg. Species = Pressure
                            //                  IOParamA = Direction
    GetSweepTime,           //Request   0x0e, no obj
    PutSweepTime,           //Response  0x0e, requires TympMsgArg. Species = Pressure
                            //                  IOParamA = Time in ms (with resolution 10ms)
    GetMotorSpeed,          //Requesst  0x0f, requires TympMsgArg. Species = Pressure
                            //                  IOParamA = Size
    PutMotorSpeed,          //Response  0x0f, requires TympMsgArg. Species = Pressure
                            //                  IOParamA = Size, IOParamB = Speed
    SetMotorSpeed,          //Command   0x0f, requires TympMsgArg. Species = Pressure
                            //                  IOParamA = Size, IOParamB = Speed
    HomePump,               //Command   0x10, no obj
    GetConfigParam,         //Request   0x10, requires TympMsgArg. Species = Eng. IOParamA = ConfigParam
    PutConfigParam,         //Resposne  0x10, requires TympMsgArg. Species = Eng. 
                            //                  IOParamA = ConfigParam, IOParamB = Data
    SetConfigParam,         //Command   0x11, requires TympMsgArg. Species = Eng. 
                            //                  IOParamA = ConfigParam, IOParamB = Data
    InitNonVolatileMemory,  //Command   0x12, requires TympMsgArg. Species = Eng. IOParamA = memory to init
    ClearErrorFlag,         //Command   0x13, no obj
}
