use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmBATTERY:Downcast{}

#[derive(Clone,Debug)]
pub enum FmBATTERYSpecies{
	SpSTANDARD,		//0, data
	SpSERVICE,		//1, data
	SpSELECTOR,		//1, param
}
impl ObjectSpecies for FmBATTERYSpecies{}

impl FmBATTERY for CBatteryDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CBatteryDStd{
	batteryvoltage:i16,
	batterycurrent:u16,
	batterylevel:BatteryLevel,
	batterystatus:BatteryStatus,
	acconnected:bool,
	batterychemistry:BatteryChemistry,
	batterypercentage:u16,			//Current battery charge percentage (1-100)
	fullchargetime:u16,				//Average time to full charge in minutes
	batterylife:u16,					//% of battery life remaining (1-100)
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum BatteryLevel{
	#[default]
	CRITICALLY_LOW_CHARGE,	//0
	LOW_CHARGE,				//1
	MEDIUM_CHARGE,			//2
	FULL_CHARGE,			//3
}

#[derive(Clone,Copy,Debug,Default)]
pub enum BatteryStatus{
	#[default]
	NOT_CHARGING,	//0
	FAST_CHARGE,    //1
	TRICKLE_CHARGE, //2
	NO_BATTERY,     //3
}

#[derive(Clone,Copy,Debug,Default)]
pub enum BatteryChemistry{
	#[default]
	NONE,		//0
	UNKNOWN,    //1
	ALKALINE,   //2
	NiMH,       //3
	LI_ION,     //4
}

impl CBatteryDStd {
	pub fn new() -> Self { todo!() }
}
impl Object for CBatteryDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmBATTERY }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmBATTERYSpecies::CBatteryDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmBATTERY for CBatteryDService{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CBatteryDService{
	identifier:uint32,
	data:uint8,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CBatteryDService {
	fn new() -> Self { todo!() }
}
impl Object for CBatteryDService{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmBATTERY }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmBATTERYSpecies::CBatteryDService) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmBATTERY for CBatteryPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CBatteryPSelector{
	speciesselector:uint32,
	ioparametera:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CBatteryPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CBatteryPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmBATTERY }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmBATTERYSpecies::CBatteryPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

