use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmBODYMASSINDEX:Downcast{}

#[derive(Clone,Debug)]
pub enum FmBODYMASSINDEXSpecies{
	SpSTANDARD, //0
}
impl ObjectSpecies for FmBODYMASSINDEXSpecies{}

impl FmBODYMASSINDEX for CBMIDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CBMIDStd{
	stime:chrono::NaiveDateTime,
	status:Status,
	weight:u32,
	height:u32,
	bodymassindex:u32,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	bodymassindexext:f32,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	COMPUTED,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

impl CBMIDStd {
	pub fn new() -> Self { todo!() }
}
impl Object for CBMIDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmBODYMASSINDEX }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmBODYMASSINDEXSpecies::SpSTANDARD) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

