use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject, FmNUMERIC_objects};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmCO2:Downcast{}

#[derive(Clone,Debug)]
pub enum FmCO2Species{
	SpDATA,			//1, data
	SpCALIBRATION,	//11, config
	SpSTANDARD,		//0
	SpMSG_ARGUMENTS,//1, param
}
impl ObjectSpecies for FmCO2Species{}

impl FmCO2 for CCO2DData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CCO2DData{
	status:DataStatus,
	extstatus:DataExtStatus,
	source:Source,
	mode:Mode,
	method:DataMethod,
	etco2:FmNUMERIC_objects::CNumDStd,
	fico2:FmNUMERIC_objects::CNumDStd,
	rr:FmNUMERIC_objects::CNumDStd,
	ipi:FmNUMERIC_objects::CNumDStd,
	s_etco2:FmNUMERIC_objects::CNumDStd,
	s_fico2:FmNUMERIC_objects::CNumDStd,
	contco2:FmNUMERIC_objects::CNumDStd,
	s_density_5:FmNUMERIC_objects::CNumDStd,
	s_density_10:FmNUMERIC_objects::CNumDStd,
	s_density_30:FmNUMERIC_objects::CNumDStd,
	apneaindex_total:FmNUMERIC_objects::CNumDStd,
	apneaindex_10_19:FmNUMERIC_objects::CNumDStd,
	apneaindex_20_29:FmNUMERIC_objects::CNumDStd,
	apneaindex_30:FmNUMERIC_objects::CNumDStd,
	desatcount:FmNUMERIC_objects::CNumDStd,
	stime:chrono::NaiveDateTime,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum DataStatus{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum DataExtStatus{
	#[default]
	INITIALIZATION,
	SFM_REQUIRED,
	SFM_IN_PROGRESS,
	PURGING_IN_PROGRESS,
	FILTER_LINE_NO_CONNECT,
	CHECK_CALIBRATION,
	CHECK_FLOW,
	OCCLUSION_IN_GAS_LINE,
	TEMPERATURE_OUT_OF_RANGE,
	PUMP_OFF,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	BREATH_DETECTED,
	INTERVAL,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum DataMethod{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

impl CCO2DData {
	pub fn new() -> Self { todo!() }
}
impl Object for CCO2DData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmCO2 }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmCO2Species::SpDATA) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmCO2 for CCO2CCal{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CCO2CCal{
	status:CalStatus,
	method:CalMethod,
	gasconcentration:u8,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum CalStatus{
	#[default]
	IN_PROGRESS,
	NOT_READY,
	CALCULATING,
	COMPLETE_ERROR,
	COMPLETE_SUCCESS,
	USER_ABORT,
	MEASUREMENT_ERROR,
	AZ_FAIL,
	GAS_CONCENTRATION_CHECK,
	GAS_FLOW_UNSTABLE,
	CALIBRATION_FAILURE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum CalMethod{
	#[default]
	NONE,
	REGULAR_CALIBRATION,
	SPECIAL_FACTORY_CALIBRATION,
	CALIBRATION_CHECK,
}

impl CCO2CCal {
	pub fn new() -> Self { todo!() }
}
impl Object for CCO2CCal{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmCO2 }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmCO2Species::SpCALIBRATION) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmCO2 for CCO2CStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CCO2CStd{
	displayunits:DisplayUnits,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum DisplayUnits{
	#[default]
	UNITS_MMHG,
	UNITS_KPA,
}

impl CCO2CStd {
	pub fn new() -> Self { todo!() }
}
impl Object for CCO2CStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmCO2 }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmCO2Species::SpSTANDARD) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmCO2 for CCO2PMsgArg{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CCO2PMsgArg{
	ioparametera:u16,
	ioparameterb:u16,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CCO2PMsgArg {
	fn new() -> Self { todo!() }
}
impl Object for CCO2PMsgArg{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmCO2 }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmCO2Species::SpMSG_ARGUMENTS) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

