use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmCOMMAND_LINE:std::fmt::Debug + Downcast{}

#[derive(Clone,Debug)]
pub enum FmCOMMAND_LINESpecies{
	SpDISPATCH,		//1, Extended
	SpSTANDARD,		//0
	SpANSI,			//1, Data
	SpUNICODE,		//2, Data
	SpDESCRIPTION,	//1, config
}
impl ObjectSpecies for FmCOMMAND_LINESpecies{}

impl FmCOMMAND_LINE for CCLIDExtDispatch{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CCLIDExtDispatch{
	commandhandle:u16,
	viewhandle:u16,
	flags:u8,
	errorcode:ErrorCode,
	container:&'static dyn FmCOMMAND_LINE,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ErrorCode{
	#[default]
	NONE,
	UNSUPPORTED,
	CLASS_SYNTAX,
	COMMAND_SYNTAX,
	PARAMETER_SYNTAX,
}

impl CCLIDExtDispatch {
	pub fn new() -> Self { todo!() }
}
impl Object for CCLIDExtDispatch{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmCOMMAND_LINE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATAEXT }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmCOMMAND_LINESpecies::SpDISPATCH) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmCOMMAND_LINE for CCLIDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CCLIDStd{
	dataformat:DataFormat,
	endianess:Endianess,
	data:u8,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum DataFormat{
	#[default]
	NONE,
	ANSI,
	UNICODE,
	BINARY,
	MOIB,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Endianess{
	#[default]
	NONE,
	BIG_ENDIAN,
	LITTLE_ENDIAN,
}

impl CCLIDStd {
	pub fn new() -> Self { todo!() }
}
impl Object for CCLIDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmCOMMAND_LINE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmCOMMAND_LINESpecies::SpSTANDARD) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmCOMMAND_LINE for CCLIDAnsi{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CCLIDAnsi{
	argcount:u32,
	arguments:String,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CCLIDAnsi {
	pub fn new() -> Self { todo!() }
}
impl Object for CCLIDAnsi{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmCOMMAND_LINE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmCOMMAND_LINESpecies::SpANSI) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmCOMMAND_LINE for CCLIDUnicode{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CCLIDUnicode{
	argcount:u32,
	arguments:String,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CCLIDUnicode {
	pub fn new() -> Self { todo!() }
}
impl Object for CCLIDUnicode{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmCOMMAND_LINE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmCOMMAND_LINESpecies::SpUNICODE) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmCOMMAND_LINE for CCLICDesc{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CCLICDesc{
	dataformat:DescDataFormat,
	delimiter:String,
	startsequence:String,
	stopsequence:String,
	maxcharacterlimit:u32,
	charsetpage:u16,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum DescDataFormat{
	#[default]
	NONE,
	ANSI,
	UNICODE,
}

impl CCLICDesc {
	pub fn new() -> Self { todo!() }
}
impl Object for CCLICDesc{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmCOMMAND_LINE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmCOMMAND_LINESpecies::SpDESCRIPTION) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

