use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmCUSTOM:Downcast{}

#[derive(Clone,Debug)]
pub enum FmCUSTOMSpecies{
	SpDATA,
	SpMODULEINFO,
}
impl ObjectSpecies for FmCUSTOMSpecies{}

impl FmCUSTOM for CCustomDData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CCustomDData{
	stime:chrono::NaiveDateTime,
	status:Status,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	value:widechar,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

pub impl CCustomDData {
	fn new() -> Self { todo!() }
}
impl Object for CCustomDData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmCUSTOM }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmCUSTOMSpecies::CCustomDData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmCUSTOM for CCustomDModuleInfo{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CCustomDModuleInfo{
	hash:ansichar,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CCustomDModuleInfo {
	fn new() -> Self { todo!() }
}
impl Object for CCustomDModuleInfo{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmCUSTOM }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmCUSTOMSpecies::CCustomDModuleInfo) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

