use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmDATATRANSFER:Downcast{}

#[derive(Clone,Debug)]
pub enum FmDATATRANSFERSpecies{
	SpSTANDARD,
	SpDESCRIPTION,
}
impl ObjectSpecies for FmDATATRANSFERSpecies{}

impl FmDATATRANSFER for CDataTransferCDesc{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDataTransferCDesc{
	initseqnumber:u32,
	totaldatasize:u32,
	datapacketsize:u16,
	role:Role,
	identifier:u32,
	descriptor:&'static dyn Object,
	windowsize:u16,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Role{
	#[default]
	NONE,
	DATA_RECIPIENT,
	DATA_SOURCE,
}

impl CDataTransferCDesc {
	pub fn new() -> Self { todo!() }
}
impl Object for CDataTransferCDesc{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDATATRANSFER }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDATATRANSFERSpecies::CDataTransferCDesc) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

