use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmDEVICE:Downcast{}

#[derive(Clone,Debug)]
pub enum FmDEVICESpecies{
	SpSTANDARD,				//0
	SpHOST,                 //1  , data
	SpSENSOR,               //2  , data
	SpPOST,                 //3  , data
	SpINVENTORY,            //4  , data
	SpALARM_INFO,           //5  , data
	SpALARM_STATE,          //6  , data
	SpTELEMETRY,            //7  , data
	SpSTATUS,               //8  , data
	SpSETTINGSXFERSUPPORT,  //9  , data
	SpSETTINGSXFEROPTIONS,  //10 , data
	SpSETTINGSXFERSTATUS,   //11 , data
	SpSETTINGSDESCRIPTION,  //12 , data
	SpUPDATESTATUS,         //13 , data
	SpSWUPDATESTATUS,       //14 , data
	SpSWUPDATEDOWNLOAD,     //15 , data
	SpSWUPDATEHWMODULES,    //16 , data
	SpALL,                  //255, data
	SpAUDIO,                //1,   config
	SpLCD,                  //2,   config
	SpCALIBRATION,          //3,   config
	SpTIME,                 //4,   config
	SpALARM,                //5,   config
	SpALARM_MASK,           //6,   config
	SpSERVICEMONITOR,       //7,   config
	SpSERVICEMONITORINFO,   //8,   config
	SpMSG_ARGUMENTS,        //0,   param
	SpRESULT,               //1,   param
	SpSELECTOR,             //2,   param
}
impl ObjectSpecies for FmDEVICESpecies{}

impl FmDEVICE for CDeviceDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDStd{
	datetime:chrono::NaiveDateTime,
	runtime:u32,
	modelname:String,
	serialnumber:String,
	deviceguid:[u8;16],
	modelnumber:String,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceDStd {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDevDPOST{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDevDPOST{
	deviceguid:u8,
	sourceid:String,
	poststatus:POSTStatus,
	errorlist:&'static dyn Object,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum POSTStatus{
	#[default]
	NONE,
	IN_PROGRESS,
	COMPLETE,
	FAILURE,
}

impl CDevDPOST {
	pub fn new() -> Self { todo!() }
}
impl Object for CDevDPOST{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDevDPOST) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDInventory{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDInventory{
	deviceguid:u8,
	modelname:String,
	modelnumber:String,
	serialnumber:String,
	softwareversion:String,
	ipaddress:u8,
	macaddress:u8,
	accesspointmacaddress:u8,
	ssid:String,
	rssi:i32,
	connectionmode:ConnectionMode,
	channel:u16,
	wiredmacaddress:u8,
	wirelessmacaddress:u8,
	wirelessfeatures:u32,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ConnectionMode{
	#[default]
	NONE,
	ETHERNET,
	WIRELESS,
}

impl CDeviceDInventory {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDInventory{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDInventory) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDAlarmInfo{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDAlarmInfo{
	rank:u16,
	r#type:Type,
	responsetype:ResponseType,
	alarmdescription:&'static dyn Object,
	holdoffallowed:bool,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Type{
	#[default]
	NONE,
	TECHNICAL,
	PHYSIOLOGICAL,
	INFORMATIONAL,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ResponseType{
	#[default]
	NONE,
	DISMISSIBLE,
	ACKNOWLEDGEABLE,
	PERSISTENT,
}

impl CDeviceDAlarmInfo {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDAlarmInfo{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDAlarmInfo) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDAlarmState{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDAlarmState{
	alarming:bool,
	audiooff:bool,
	audiooffcountdown:u16,
	alarmsoff:bool,
	alarmsoffcountdown:u16,
	alarmsconditioncode:u8,
	alarmscontainer:&'static dyn Object,
	nightmode:bool,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceDAlarmState {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDAlarmState{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDAlarmState) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDTele{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDTele{
	mode:Mode,
	description:String,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	DISCONNECT,
	END_MONITORING,
	DISCHARGE,
	POWER_DOWN,
	RETRY_TIMEOUT,
	CONNECTION_REJECTED,
}

impl CDeviceDTele {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDTele{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDTele) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDStatus{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDStatus{
	status:Status,
	msgclassid:u32,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	NONE,
	COMPLETE,
	FAILURE,
}

impl CDeviceDStatus {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDStatus{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDStatus) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDSettingsXferSupport{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDSettingsXferSupport{
	supportlevel:SupportLevel,
	settingsfilelist:&'static dyn Object,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum SupportLevel{
	#[default]
	BASIC,
}

impl CDeviceDSettingsXferSupport {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDSettingsXferSupport{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDSettingsXferSupport) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDSettingsXferOptions{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDSettingsXferOptions{
	role:Role,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Role{
	#[default]
	NONE,
	DATA_RECIPIENT,
	DATA_SOURCE,
}

impl CDeviceDSettingsXferOptions {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDSettingsXferOptions{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDSettingsXferOptions) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDSettingsXferStatus{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDSettingsXferStatus{
	status:u8,
	errorlist:&'static dyn Object,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceDSettingsXferStatus {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDSettingsXferStatus{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDSettingsXferStatus) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDSettingsDesc{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDSettingsDesc{
	filename:String,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceDSettingsDesc {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDSettingsDesc{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDSettingsDesc) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDUpdateStatus{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDUpdateStatus{
	selector:&'static dyn Object,
	status:Status,
	errorlist:&'static dyn Object,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	NONE,
	COMPLETE,
	FAILURE,
	EXCEPTION,
	PROCESSING,
	HOST_READY,
}

impl CDeviceDUpdateStatus {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDUpdateStatus{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDUpdateStatus) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDSWUpdateStatus{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDSWUpdateStatus{
	status:Status,
	extstatus:ExtStatus,
	updatename:String,
	container:&'static dyn Object,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	NONE,
	COMPLETE,
	FAILURE,
	EXCEPTION,
	PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
	PENDING,
	DOWNLOAD_MANIFEST,
	DOWNLOAD_PACKAGE,
	INSTALLING,
	INSTALLING_HW_MODULES,
	DOWNLOAD_COMPLETE,
}

impl CDeviceDSWUpdateStatus {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDSWUpdateStatus{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDSWUpdateStatus) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDProgressDownloading{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDProgressDownloading{
	totalpackages:u16,
	currentpackage:u16,
	currentpackagename:String,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceDProgressDownloading {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDProgressDownloading{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDProgressDownloading) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDProgressHWModules{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDProgressHWModules{
	totalmodules:u16,
	modulelist:String,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceDProgressHWModules {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDProgressHWModules{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDProgressHWModules) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceDExtStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceDExtStd{
	devicecontainer:&'static dyn Object,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceDExtStd {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceDExtStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATAEXT }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceDExtStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceCStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceCStd{
	stime:chrono::NaiveDateTime,
	standbyinterval:u16,
	autooffinterval:u16,
	disablestandbyextpower:bool,
	disableautooffextpower:bool,
	audiobeeperenabled:bool,
	language:Language,
	displaydateformat:DisplayDateFormat,
	displaytimeformat:DisplayTimeFormat,
	supplyvoltage:SupplyVoltage,
	deviceid:String,
	charsetpage:u16,
	practicename:String,
	dstenabled:bool,
	mode:Mode,
	displayoffinterval:u16,
	disabledisplayoffextpower:bool,
	linefrequency:LineFrequency,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Language{
	#[default]
	ENGLISH,
	GERMAN,
	FRENCH,
	SPANISH,
	ITALIAN,
	PORTUGUESE,
	DUTCH,
	SWEDISH,
	DANISH,
	CHINESE,
	FINNISH,
	NORWEGIAN,
	POLISH,
	KOREAN,
	GREEK,
	CZECH,
	ESTONIAN,
	RUSSIAN,
	HUNGARIAN,
	LATVIAN,
	SLOVENIAN,
	INDONESIAN,
	ARABIC,
	CROATIAN,
	VIETNAMESE,
	BRAZILLIANPORTUGUESE,
	THAI,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum DisplayDateFormat{
	#[default]
	MM_DD_YYYY,
	DD_MM_YYYY,
	YYYY_MM_DD,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum DisplayTimeFormat{
	#[default]
	HOUR_12,
	HOUR_24,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum SupplyVoltage{
	#[default]
	SUPPLY_110V,
	SUPPLY_NON_110V,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	AC_CONNECTED,
	AC_CONNECTED_BASE,
	BATTERY,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum LineFrequency{
	#[default]
	NONE,
	50HZ,
	60HZ,
}

impl CDeviceCStd {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceCStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceCStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceCAudio{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceCAudio{
	channel:u8,
	frequency:u16,
	volume:u16,
	state:State,
	duration:u16,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum State{
	#[default]
	NONE,
	ON,
	OFF,
	MUTE,
}

impl CDeviceCAudio {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceCAudio{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceCAudio) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceCLCD{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceCLCD{
	backlight:u8,
	contrast:u8,
	lcdcontrol:LCDControl,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum LCDControl{
	#[default]
	NONE,
	RESTORE,
	PIXELS_ON,
	PIXELS_OFF,
	PIXELS_TEST_PATTERN_1,
	PIXELS_TEST_PATTERN_2,
	PIXELS_TEST_PATTERN_3,
	PIXELS_TEST_PATTERN_4,
	PIXELS_TEST_PATTERN_5,
	PIXELS_TEST_PATTERN_6,
}

impl CDeviceCLCD {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceCLCD{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceCLCD) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceCCal{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceCCal{
	caldatetime:chrono::NaiveDateTime,
	calsignature:String,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceCCal {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceCCal{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceCCal) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceCTime{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceCTime{
	datetimeutc:chrono::NaiveDateTime,
	timezoneoffset:i16,
	dstoffset:i16,
	dstineffect:bool,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceCTime {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceCTime{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceCTime) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceCAlarm{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceCAlarm{
	audiopausetime:u16,
	alarmspausetime:u16,
	maxcharacters:u16,
	alarmscontainer:&'static dyn Object,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceCAlarm {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceCAlarm{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceCAlarm) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceCAlarmMask{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceCAlarmMask{
	alarmsconditioncode:u8,
	audiomaskmode:AudioMaskMode,
	alarmmaskmode:AlarmMaskMode,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum AudioMaskMode{
	#[default]
	NONE,
	DISABLE,
	ENABLE,
	PAUSE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum AlarmMaskMode{
	#[default]
	NONE,
	DISABLE,
	ENABLE,
	PAUSE,
}

impl CDeviceCAlarmMask {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceCAlarmMask{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceCAlarmMask) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceCServiceMonitor{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceCServiceMonitor{
	callhomeinterval:u32,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceCServiceMonitor {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceCServiceMonitor{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceCServiceMonitor) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceCServiceMonitorInfo{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceCServiceMonitorInfo{
	serverip:u8,
	active:bool,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceCServiceMonitorInfo {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceCServiceMonitorInfo{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceCServiceMonitorInfo) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDevPMsgArguments{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDevPMsgArguments{
	speciesselector:u32,
	ioparametera:u32,
	ioparameterb:u32,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDevPMsgArguments {
	pub fn new() -> Self { todo!() }
}
impl Object for CDevPMsgArguments{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDevPMsgArguments) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmDEVICE for CDeviceUpdatePSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CDeviceUpdatePSelector{
	updatetoken:String,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CDeviceUpdatePSelector {
	pub fn new() -> Self { todo!() }
}
impl Object for CDeviceUpdatePSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmDEVICE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmDEVICESpecies::CDeviceUpdatePSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

