use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmERROR:Downcast{}

#[derive(Clone,Debug)]
pub enum FmERRORSpecies{
	SpERRNONE,
	SpERRWACPSTACK,
	SpERRMSGPROC,
	SpERRAPPINTERNAL,
	SpGET_ERROR,
	SpSTANDARD,
	SpDESCRIPTOR,
}
impl ObjectSpecies for FmERRORSpecies{}

impl FmERROR for CErrEStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CErrEStd{
	errclassid:u32,
	errorcode:u16,
	description:String,
	data:u8,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CErrEStd {
	pub fn new() -> Self { todo!() }
}
impl Object for CErrEStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmERROR }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnERROR }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmERRORSpecies::SpSTANDARD) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmERROR for CErrPGetError{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CErrPGetError{
	errclassid:u32,
	source:u16,
	errcode:u16,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CErrPGetError {
	pub fn new() -> Self { todo!() }
}
impl Object for CErrPGetError{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmERROR }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmERRORSpecies::SpGET_ERROR) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmERROR for CErrDExtStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CErrDExtStd{
	errorcontainer:&'static dyn FmERROR,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CErrDExtStd {
	pub fn new() -> Self { todo!() }
}
impl Object for CErrDExtStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmERROR }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATAEXT }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmERRORSpecies::SpSTANDARD) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmERROR for CErrDDesc{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CErrDDesc{
	ioparameter:u32,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CErrDDesc {
	pub fn new() -> Self { todo!() }
}
impl Object for CErrDDesc{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmERROR }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmERRORSpecies::SpDESCRIPTOR) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

