use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmFILESYS:Downcast{}

#[derive(Clone,Debug)]
pub enum FmFILESYSSpecies{
	SpFILE,
	SpFILEDESCRIPTION,
	SpSELECTOR,
}
impl ObjectSpecies for FmFILESYSSpecies{}

impl FmFILESYS for CFileSysDFil{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CFileSysDFil{
	datetime:chrono::NaiveDateTime,
	storagemedium:StorageMedium,
	filename:widechar,
	filecontents:uint8,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum StorageMedium{
	#[default]
	STANDARD,
	RAM,
	FLASH,
	SECUREDIGITALCARD,
	COMPACTFLASHCARD,
}

pub impl CFileSysDFil {
	fn new() -> Self { todo!() }
}
impl Object for CFileSysDFil{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmFILESYS }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmFILESYSSpecies::CFileSysDFil) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmFILESYS for CFileSysDFileDesc{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CFileSysDFileDesc{
	datetime:chrono::NaiveDateTime,
	size:uint32,
	storagemedium:StorageMedium,
	filename:widechar,
	attributes:Attributes,
	freespace:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum StorageMedium{
	#[default]
	STANDARD,
	RAM,
	FLASH,
	SECUREDIGITALCARD,
	COMPACTFLASHCARD,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Attributes{
	#[default]
	READONLY,
	VOLUMEROOT,
}

pub impl CFileSysDFileDesc {
	fn new() -> Self { todo!() }
}
impl Object for CFileSysDFileDesc{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmFILESYS }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmFILESYSSpecies::CFileSysDFileDesc) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmFILESYS for CFileSysPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CFileSysPSelector{
	pathname:widechar,
	startindex:uint16,
	endindex:uint16,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CFileSysPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CFileSysPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmFILESYS }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmFILESYSSpecies::CFileSysPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

