use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmHEIGHT:Downcast{}

#[derive(Clone,Debug)]
pub enum FmHEIGHTSpecies{
	SpSTANDARD,
}
impl ObjectSpecies for FmHEIGHTSpecies{}

impl FmHEIGHT for CHeightDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CHeightDStd{
	stime:chrono::NaiveDateTime,
	status:Status,
	height:uint32,
	extstatus:ExtStatus,
	mode:Mode,
	method:Method,
	source:Source,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
}

pub impl CHeightDStd {
	fn new() -> Self { todo!() }
}
impl Object for CHeightDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmHEIGHT }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmHEIGHTSpecies::CHeightDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmHEIGHT for CHeightCStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CHeightCStd{
	enabled:boolean,
	manualheight:boolean,
	heightdisplayunits:HeightDisplayUnits,
	heightdefault:uint32,
	externalport:ExternalPort,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum HeightDisplayUnits{
	#[default]
	UNITS_CENTIMETRES,
	UNITS_INCHES,
	UNITS_FEET_INCHES,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExternalPort{
	#[default]
	NONE,
	UNKNOWN,
	PORT_1,
	PORT_2,
	PORT_3,
	PORT_4,
}

pub impl CHeightCStd {
	fn new() -> Self { todo!() }
}
impl Object for CHeightCStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmHEIGHT }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmHEIGHTSpecies::CHeightCStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

