use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmHIM:Downcast{}

#[derive(Clone,Debug)]
pub enum FmHIMSpecies{
	SpSTANDARD,
	SpRESPONSE,
	SpSELECTOR,
}
impl ObjectSpecies for FmHIMSpecies{}

impl FmHIM for CHIMDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CHIMDStd{
	identifier:uint16,
	sequencenumber:uint16,
	status:uint8,
	rank:uint16,
	content:widechar,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CHIMDStd {
	fn new() -> Self { todo!() }
}
impl Object for CHIMDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmHIM }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmHIMSpecies::CHIMDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmHIM for CHIMDResponse{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CHIMDResponse{
	identifier:uint16,
	result:widechar,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CHIMDResponse {
	fn new() -> Self { todo!() }
}
impl Object for CHIMDResponse{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmHIM }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmHIMSpecies::CHIMDResponse) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmHIM for CHIMCStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CHIMCStd{
	enabled:boolean,
	unicodesupport:uint8,
	maxrows:uint16,
	maxcolumns:uint16,
	maxcontentsize:uint16,
	maxmessage:uint8,
	supportedcapabilities:widechar,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CHIMCStd {
	fn new() -> Self { todo!() }
}
impl Object for CHIMCStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmHIM }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmHIMSpecies::CHIMCStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmHIM for CHIMPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CHIMPSelector{
	identifier:uint16,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CHIMPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CHIMPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmHIM }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmHIMSpecies::CHIMPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

