use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmHR:Downcast{}

#[derive(Clone,Debug)]
pub enum FmHRSpecies{
	SpDATA,
	SpSELECTOR,
}
impl ObjectSpecies for FmHRSpecies{}

impl FmHR for CHRDData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CHRDData{
	stime:chrono::NaiveDateTime,
	status:Status,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	hr:Object::CNumDStd,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
	UNKNOWN,
	HR_FOUND,
	NO_PROBE,
	INCORRECT_PROBE,
	SEARCHING_FOR_HR,
	BAD_PROBE,
	PROBE_EXPIRED,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	NIBP,
	SPO2,
	ECG,
	PATIENTMOTION_BED_SENSOR,
	PATIENTMOTION_CHAIR_SENSOR,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
	AVERAGE_HR,
}

pub impl CHRDData {
	fn new() -> Self { todo!() }
}
impl Object for CHRDData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmHR }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmHRSpecies::CHRDData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmHR for CHRCData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CHRCData{
	hralarms:Object::CNumCStd,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CHRCData {
	fn new() -> Self { todo!() }
}
impl Object for CHRCData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmHR }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmHRSpecies::CHRCData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmHR for CHRPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CHRPSelector{
	speciesselector:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CHRPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CHRPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmHR }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmHRSpecies::CHRPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

