use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmNETWORK:Downcast{}

#[derive(Clone,Debug)]
pub enum FmNETWORKSpecies{
	SpWIRELESS_STATUS,
	SpETHERNET,
	SpLOCATIONID,
	SpWIFI,
	SpWORKSTATION,
	SpADAPTER,
	SpETHERNET_WIFI,
	SpBLUETOOTH,
	SpBLUETOOTHDEVICE,
	SpADDRESS,
	SpWEP,
	SpEAP,
	SpPSK,
	SpALL,
	SpNETWORK,
	SpSTANDARD,
}
impl ObjectSpecies for FmNETWORKSpecies{}

impl FmNETWORK for CNetworkDWirelessStatus{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkDWirelessStatus{
	status:Status,
	accesspointmacaddress:uint8,
	rssi:int32,
	channel:uint16,
	linkquality:uint8,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

pub impl CNetworkDWirelessStatus {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkDWirelessStatus{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkDWirelessStatus) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkCEth{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkCEth{
	fixedipaddress:byte,
	defaultgateway:byte,
	subnetmask:byte,
	dnsarray1:byte,
	dnsarray2:byte,
	dhcpenabled:boolean,
	autodnsenabled:boolean,
	wantimeoutinterval:uint16,
	networkmode:NetworkMode,
	servername:widechar,
	networkid:widechar,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum NetworkMode{
	#[default]
	WIRED,
	WIRELESS,
}

pub impl CNetworkCEth {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkCEth{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkCEth) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkCLocID{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkCLocID{
	locationid:ansichar,
	unitid:ansichar,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNetworkCLocID {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkCLocID{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkCLocID) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkCWIFI{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkCWIFI{
	enabled:boolean,
	external:boolean,
	installedmodule:InstalledModule,
	security:Security,
	channel:Channel,
	mode:Mode,
	externalport:ExternalPort,
	authenticationmode:AuthenticationMode,
	authenticationtype:AuthenticationType,
	encryptionkeysize:EncryptionKeySize,
	autogeneratekey:boolean,
	ssid:widechar,
	ieee_802_1xenabled:boolean,
	enterkey:widechar,
	keyindex:widechar,
	channel5ghz:uint16,
	radiomode:RadioMode,
	dfsenabled:boolean,
	authenticationcontainer:Object::ANY_CLASS,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum InstalledModule{
	#[default]
	NONE,
	UNKNOWN,
	DPAC,
	LAMARR,
	NEWMAR,
	BOOTLOADER,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Security{
	#[default]
	NONE,
	WPA,
	WEP,
	AES,
	TKIP,
	DISABLED,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Channel{
	#[default]
	NONE,
	WI_FI_CHANNEL_1,
	WI_FI_CHANNEL_2,
	WI_FI_CHANNEL_3,
	WI_FI_CHANNEL_4,
	WI_FI_CHANNEL_5,
	WI_FI_CHANNEL_6,
	WI_FI_CHANNEL_7,
	WI_FI_CHANNEL_8,
	WI_FI_CHANNEL_9,
	WI_FI_CHANNEL_10,
	WI_FI_CHANNEL_11,
	WI_FI_CHANNEL_12,
	WI_FI_CHANNEL_13,
	WI_FI_CHANNEL_14,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	WI_FI_DHCP,
	WI_FI_FIXED_IP,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExternalPort{
	#[default]
	NONE,
	UNKNOWN,
	PORT_1,
	PORT_2,
	PORT_3,
	PORT_4,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum AuthenticationMode{
	#[default]
	NONE,
	OPEN,
	SHARED,
	WPA,
	WPA_PSK,
	WPA2_ENTERPRISE,
	WPA2_PSK,
	WEP,
	WPA_ENTERPRISE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum AuthenticationType{
	#[default]
	NONE,
	EAP,
	EAP_TLS,
	EAP_MD5_CHAP,
	EAP_MS_CHAP_V2,
	PEAP,
	PEAP_TLS,
	PEAP_MS_CHAP_V2,
	LEAP,
	SMART_CARD_OR_OTHER,
	EAP_TTLS,
	EAP_FAST,
	WEP_64,
	WEP_128,
	PEAP_GTC,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum EncryptionKeySize{
	#[default]
	NONE,
	64_BIT_KEY,
	128_BIT_KEY,
	256_BIT_KEY,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum RadioMode{
	#[default]
	NONE,
	ABGN,
	ABG,
	BGN,
	AN,
	BG,
	AB,
	B,
	A,
}

pub impl CNetworkCWIFI {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkCWIFI{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkCWIFI) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkCWkst{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkCWkst{
	connectiontype:ConnectionType,
	remoteport:uint16,
	remoteipaddress:byte,
	remotenetworkname:widechar,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ConnectionType{
	#[default]
	NONE,
	WIREDNETWORK,
	WIRELESSNETWORK,
	USB_ACTIVESYNC,
}

pub impl CNetworkCWkst {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkCWkst{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkCWkst) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkCAdapter{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkCAdapter{
	physicaladdress:byte,
	description:widechar,
	mediastate:MediaState,
	configcontainer:Object::ANY_CLASS,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum MediaState{
	#[default]
	NONE,
	DISABLED,
	DISCONNECTED,
	CONNECTED,
}

pub impl CNetworkCAdapter {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkCAdapter{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkCAdapter) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkCBTooth{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkCBTooth{
	state:State,
	profile:Profile,
	version:ansichar,
	timeout:uint16,
	power:uint8,
	connectiontype:ConnectionType,
	leasetime:uint32,
	connections:Object::ANY_CLASS,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum State{
	#[default]
	NONE,
	UNKNOWN,
	ENABLED_PAIRED,
	ENABLED_NOT_PAIRED,
	DISABLED_PAIRED,
	DISABLED_NOT_PAIRED,
	IN_PROCESS,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Profile{
	#[default]
	NONE,
	UNKNOWN,
	A2DDP,
	AVRCP,
	BIP,
	BPP,
	FTP,
	GAVDP,
	GAP,
	GOEP,
	HCRP,
	HFP,
	HID,
	OPP,
	PAN,
	SPP,
	SIM,
	VDP,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ConnectionType{
	#[default]
	NONE,
	UNKNOWN,
	TEMPORARY,
	PERMANENT,
	FACTORY,
}

pub impl CNetworkCBTooth {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkCBTooth{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkCBTooth) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkCBToothDev{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkCBToothDev{
	physicaladdress:byte,
	friendlyname:widechar,
	connectable:boolean,
	discoverable:boolean,
	iosseedname:ansichar,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNetworkCBToothDev {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkCBToothDev{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkCBToothDev) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkCAddress{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkCAddress{
	ipaddress:byte,
	port:uint16,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNetworkCAddress {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkCAddress{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkCAddress) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkCWEP{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkCWEP{
	wepkeysize:uint16,
	wepkey1:widechar,
	wepkey2:widechar,
	wepkey3:widechar,
	wepkey4:widechar,
	wepindexkey:uint16,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNetworkCWEP {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkCWEP{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkCWEP) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkCEAP{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkCEAP{
	eapusername:widechar,
	eappassword:widechar,
	eapauthenticationserver:uint16,
	eapinnerauthentication:uint16,
	eapvalidateserverenabled:boolean,
	eappacprovisiontype:uint16,
	eaproamingtype:EAPRoamingType,
	eapouterauthentication:uint16,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum EAPRoamingType{
	#[default]
	NONE,
	PMK_CACHING,
	OKC,
	80211R,
	CCKM,
}

pub impl CNetworkCEAP {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkCEAP{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkCEAP) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkCPSK{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkCPSK{
	pskkey:widechar,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNetworkCPSK {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkCPSK{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkCPSK) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkPSelector{
	speciesselector:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNetworkPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNETWORK for CNetworkDExtStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNetworkDExtStd{
	networkcontainer:Object::ANY_CLASS,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNetworkDExtStd {
	fn new() -> Self { todo!() }
}
impl Object for CNetworkDExtStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNETWORK }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATAEXT }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNETWORKSpecies::CNetworkDExtStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

