use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmNIBP:Downcast{}

#[derive(Clone,Debug)]
pub enum FmNIBPSpecies{
	SpSTANDARD,
	SpDATA,
	SpENGCHUNK,
	SpEVENTCHUNK,
	SpHARDWARESTATE,
	SpLOCK,
	SpRAWDATA,
	SpPRESSUREDATA,
	SpSTATUS,
	SpDATAMODIFIED,
	SpCALIBRATION,
	SpSELECTOR,
	SpCHUNK_SELECTOR,
}
impl ObjectSpecies for FmNIBPSpecies{}

impl FmNIBP for CNIBPDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPDStd{
	stime:chrono::NaiveDateTime,
	status:Status,
	systolic:int16,
	diastolic:int16,
	map:int16,
	hr:uint16,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	CVSM,
	MODB,
	MODF,
	POEM,
	MODG,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
	ADULT,
	NEONATAL,
	PEDIATRIC,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
	FAST_BP,
	STEP_BP,
	NEONATAL_STEP_BP,
	LINEAR_DEFLATION_BP,
	AVERAGE_BP,
}

pub impl CNIBPDStd {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPDData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPDData{
	status:Status,
	systolic:Object::CNumDStd,
	diastolic:Object::CNumDStd,
	map:Object::CNumDStd,
	pr:Object::CNumDStd,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	RD_OK,
	RD_ABORT,
	RD_NOT_AVAIL,
	RD_SPLAT,
	RD_ARTIFACT,
	RD_LOW_INFLAT,
	RD_KINKED_TUBE,
	RD_EARLY_DUMP,
	RD_NOT_ENOUGH_DATA,
	RD_PATIENT_INVALID,
	RD_REINF_TOO_LATE,
	RD_REINF_LIMIT,
	RD_BLEED,
	RD_LEAK,
	RD_SAFETY_FAULT,
	RD_ZERO_FAULT,
}

pub impl CNIBPDData {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPDData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPDData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPDChunk{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPDChunk{
	tag:int8,
	chunkid:uint16,
	chunkindex:uint16,
	chunkdata:uint8,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNIBPDChunk {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPDChunk{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPDChunk) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPDHState{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPDHState{
	hardware:hardware,
	state:int16,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum hardware{
	#[default]
	PRIM_VALVE,
	SEC_VALVE,
	PUMP,
}

pub impl CNIBPDHState {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPDHState{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPDHState) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPDLock{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPDLock{
	lock:lock,
	state:int8,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum lock{
	#[default]
	AMB_TEMP_CHECK,
	PRIM_SAFETY_CHECK,
	SAFE_SAFETY_CHECK,
	SINGLE_LUMEN_CHECK,
	AD_CHANNEL,
	FAILSAFE_TEST_MODE,
}

pub impl CNIBPDLock {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPDLock{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPDLock) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPDRData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPDRData{
	raw5v:uint32,
	raw1v:uint32,
	raw2v:uint32,
	raw4v:uint32,
	rawbat:int16,
	rawambtemp:uint32,
	rawprimtrans:uint32,
	rawsectrans:int16,
	rawsecbat:int16,
	rawseccorr:int16,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNIBPDRData {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPDRData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPDRData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPDPData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPDPData{
	primpress:int16,
	primpressnoise:int16,
	primazerooffset:int16,
	secpress:int16,
	secpressnoise:int16,
	secazerooffset:int16,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNIBPDPData {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPDPData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPDPData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPDStatus{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPDStatus{
	modulestate:moduleState,
	error:uint16,
	hardwarestates:uint16,
	safetystates:safetyStates,
	cuffpressure:int16,
	batvoltage:int16,
	ambtemp:int16,
	statustype:statusType,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum moduleState{
	#[default]
	IDLE,
	AUTOZERO,
	INFLATING,
	STABILIZING,
	BLEEDING,
	DUMPING,
	REPUMP,
	ERROR,
	BOOT,
	POST,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum safetyStates{
	#[default]
	belowResidual,
	longAutoTimerClear,
	statAutoTimerClear,
	failsafeFault,
	autoZeroFault,
	transducerFault,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum statusType{
	#[default]
	NONE,
	PRIMARY,
	SECONDARY,
}

pub impl CNIBPDStatus {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPDStatus{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPDStatus) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPDDataModified{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPDDataModified{
	nibpdata:Object::CNIBPDData,
	status:Status,
	stime:chrono::NaiveDateTime,
	mode:Mode,
	patientposition:PatientPosition,
	cuffsize:CuffSize,
	cufflocation:CuffLocation,
	source:Source,
	method:Method,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
	ADULT,
	NEONATAL,
	PEDIATRIC,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum PatientPosition{
	#[default]
	NONE,
	UNKNOWN,
	LYING,
	SITTING,
	STANDING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum CuffSize{
	#[default]
	NONE,
	UNKNOWN,
	NEONATAL_1,
	NEONATAL_2,
	NEONATAL_3,
	NEONATAL_4,
	NEONATAL_5,
	SMALL_INFANT,
	INFANT,
	SMALL_CHILD,
	CHILD,
	SMALL_ADULT,
	ADULT,
	ADULT_LONG,
	LARGE_ADULT,
	LARGE_ADULT_LONG,
	THIGH,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum CuffLocation{
	#[default]
	NONE,
	UNKNOWN,
	LEFT_ARM,
	RIGHT_ARM,
	LEFT_LEG,
	RIGHT_LEG,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	CVSM,
	MODB,
	MODF,
	POEM,
	MODG,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
	FAST_BP,
	STEP_BP,
	NEONATAL_STEP_BP,
	LINEAR_DEFLATION_BP,
	AVERAGE_BP,
}

pub impl CNIBPDDataModified {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPDDataModified{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPDDataModified) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPCStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPCStd{
	displayunits:DisplayUnits,
	mapdisplay:MAPDisplay,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum DisplayUnits{
	#[default]
	NIBP_MMHG,
	NIBP_KPA,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum MAPDisplay{
	#[default]
	DISABLE,
	ENABLE,
}

pub impl CNIBPCStd {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPCStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPCStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPCData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPCData{
	dfltadulttargpress:int16,
	dfltpedtargpress:int16,
	dfltneotargpress:int16,
	pedmaxpresslimit:int16,
	residualmaxpresslimit:int16,
	dfltpatientmode:dfltPatientMode,
	dfltadultalg:int8,
	dfltpedalg:int8,
	dfltneoalg:int8,
	saoenabled:boolean,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum dfltPatientMode{
	#[default]
	ADULT,
	PEDIATRIC,
	NEONATE,
}

pub impl CNIBPCData {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPCData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPCData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPCCal{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPCCal{
	caltype:calType,
	calsignature:int32,
	caldatetime:chrono::NaiveDateTime,
	modconfig:int16,
	spancorr:int16,
	zerocorr:int16,
	battcal:int16,
	tempcal:int16,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum calType{
	#[default]
	NONE,
	PRIMARY,
	SECONDARY,
}

pub impl CNIBPCCal {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPCCal{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPCCal) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPPData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPPData{
	targpress:int16,
	patientmode:patientMode,
	cyclemode:cycleMode,
	algorithm:algorithm,
	disableneocheck:int8,
	firstreading:boolean,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum patientMode{
	#[default]
	DEFAULT,
	ADULT,
	PEDIATRIC,
	NEONATE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum cycleMode{
	#[default]
	MANUAL,
	LONG_AUTO,
	STAT,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum algorithm{
	#[default]
	DEFAULT,
	FAST_BP,
	STEP_BP,
	NEO_BP,
}

pub impl CNIBPPData {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPPData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPPData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPPSelector{
	value:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNIBPPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPPChunkSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPPChunkSelector{
	chunk:uint32,
	index:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNIBPPChunkSelector {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPPChunkSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPPChunkSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNIBP for CNIBPDExtStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNIBPDExtStd{
	container:Object::ANY_CLASS,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CNIBPDExtStd {
	fn new() -> Self { todo!() }
}
impl Object for CNIBPDExtStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNIBP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATAEXT }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNIBPSpecies::CNIBPDExtStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

