use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmNUMERIC:Downcast{}

#[derive(Clone,Debug)]
pub enum FmNUMERICSpecies{
	SpSTANDARD,				//0
	SpNIBPSYS,              //1
	SpNIBPDIA,              //2
	SpNIBPMAP,              //3
	SpNIBPPR,               //4
	SpIBPSYS,               //5
	SpIBPDIA,               //6
	SpIBPMAP,               //7
	SpIBPPR,                //8
	SpSPO2PR,               //9
	SpSPO2SAT,              //10
	SpSPO2PI,               //11
	SpCO2RR,                //12
	SpCO2ET,                //13
	SpCO2FI,                //14
	SpRR,                   //15
	SpPR,                   //16
	SpIPI,                  //17
	SpTHB,                  //18
	SpTHB_mmol_L,           //19
	SpPAIN,                 //20
	SpPATIENTMOTION,        //21
	SpATTEMPTEDTURNCOUNT,   //22
	SpTURNTIMER,            //23
	SpFLOAT,                //31
	SpTEMP,                 //32
	SpLONG,                 //127
	SpWEIGHT,               //128
}
impl ObjectSpecies for FmNUMERICSpecies{}

impl FmNUMERIC for CNumDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNumDStd{
	value:i16,
	statusbits:Vec<StatusBits>,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum StatusBits{
	#[default]
	ValidRange,
	OverRange,
	UnderRange,
	UndeterminedRange,
	NoAlarm,
	UpperLimitAlarm,
	LowerLimitAlarm,
	OtherAlarm,
	NumericActive,
	Continuous,
	SpotOld,
	SpotRecent,
	SpotNew,
	Dubious,
}

impl CNumDStd {
	pub fn new() -> Self { todo!() }
}
impl Object for CNumDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNUMERIC }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNUMERICSpecies::CNumDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNUMERIC for CNumDFloat{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNumDFloat{
	value:f32,
	statusbits:Vec<StatusBits>,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CNumDFloat {
	pub fn new() -> Self { todo!() }
}
impl Object for CNumDFloat{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNUMERIC }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNUMERICSpecies::CNumDFloat) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNUMERIC for CNumDLong{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNumDLong{
	value:i32,
	statusbits:Vec<StatusBits>,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CNumDLong {
	pub fn new() -> Self { todo!() }
}
impl Object for CNumDLong{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNUMERIC }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNUMERICSpecies::CNumDLong) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmNUMERIC for CNumPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CNumPSelector{
	speciesselector:u8,
	ioparametera:u32,
	#[getter(skip)]
	raw_object:RawObject,
}

impl CNumPSelector {
	pub fn new() -> Self { todo!() }
}
impl Object for CNumPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmNUMERIC }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmNUMERICSpecies::CNumPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

