use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmPAIN:Downcast{}

#[derive(Clone,Debug)]
pub enum FmPAINSpecies{
	SpSTANDARD,
}
impl ObjectSpecies for FmPAINSpecies{}

impl FmPAIN for CPainDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CPainDStd{
	stime:chrono::NaiveDateTime,
	status:Status,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	painindex:uint16,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

pub impl CPainDStd {
	fn new() -> Self { todo!() }
}
impl Object for CPainDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmPAIN }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmPAINSpecies::CPainDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

