use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmPATIENTMOTION:Downcast{}

#[derive(Clone,Debug)]
pub enum FmPATIENTMOTIONSpecies{
	SpDATA,
	SpTURN,
	SpMSG_ARGUMENTS,
	SpSELECTOR,
}
impl ObjectSpecies for FmPATIENTMOTIONSpecies{}

impl FmPATIENTMOTION for CPMDData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CPMDData{
	stime:chrono::NaiveDateTime,
	status:Status,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	motionlevel:Object::CNumDStd,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
	MOTION_FOUND,
	NO_PROBE,
	INCORRECT_PROBE,
	SEARCHING_FOR_MOTION,
	BAD_PROBE,
	PROBE_EXPIRED,
	OUT_OF_BED,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	PATIENTMOTION_BED_SENSOR,
	PATIENTMOTION_CHAIR_SENSOR,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

pub impl CPMDData {
	fn new() -> Self { todo!() }
}
impl Object for CPMDData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmPATIENTMOTION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmPATIENTMOTIONSpecies::CPMDData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmPATIENTMOTION for CPMDTurn{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CPMDTurn{
	stime:chrono::NaiveDateTime,
	status:Status,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	totalconfirmedturns:uint16,
	totalunconfirmedturns:uint16,
	attemptedturns:uint16,
	turntimer:Object::CNumDStd,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
	CONFIRMED_TURN_FOUND,
	UNCONFIRMED_TURN_FOUND,
	NO_PROBE,
	INCORRECT_PROBE,
	SEARCHING_FOR_TURN,
	BAD_PROBE,
	PROBE_EXPIRED,
	OUT_OF_BED,
	ATTEMPTED_TURN_FOUND,
	TURN_INTERVAL_EXCEEDED,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	PATIENTMOTION_BED_SENSOR,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

pub impl CPMDTurn {
	fn new() -> Self { todo!() }
}
impl Object for CPMDTurn{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmPATIENTMOTION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmPATIENTMOTIONSpecies::CPMDTurn) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmPATIENTMOTION for CPMCData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CPMCData{
	bedexitsensitivity:int16,
	chairexitsensitivity:int16,
	bedexitalarmenabled :boolean,
	chairexitalarmenabled :boolean,
	motionsource:MotionSource,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum MotionSource{
	#[default]
	NONE,
	UNKNOWN,
	PATIENTMOTION_BED_SENSOR,
	PATIENTMOTION_CHAIR_SENSOR,
}

pub impl CPMCData {
	fn new() -> Self { todo!() }
}
impl Object for CPMCData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmPATIENTMOTION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmPATIENTMOTIONSpecies::CPMCData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmPATIENTMOTION for CPMPMsgArg{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CPMPMsgArg{
	ioparametera:uint32,
	ioparameterb:int32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CPMPMsgArg {
	fn new() -> Self { todo!() }
}
impl Object for CPMPMsgArg{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmPATIENTMOTION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmPATIENTMOTIONSpecies::CPMPMsgArg) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmPATIENTMOTION for CPMPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CPMPSelector{
	speciesselector:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CPMPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CPMPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmPATIENTMOTION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmPATIENTMOTIONSpecies::CPMPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

