use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmPATIENT:Downcast{}

#[derive(Clone,Debug)]
pub enum FmPATIENTSpecies{
	SpSTANDARD,
	SpETHNIC,
	SpSELECTOR,
}
impl ObjectSpecies for FmPATIENTSpecies{}

impl FmPATIENT for CPatientDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CPatientDStd{
	stime:chrono::NaiveDateTime,
	status:Status,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	identifier:ansichar,
	identifierext:widechar,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	BARCODE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

pub impl CPatientDStd {
	fn new() -> Self { todo!() }
}
impl Object for CPatientDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmPATIENT }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmPATIENTSpecies::CPatientDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmPATIENT for CPatientDEthnic{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CPatientDEthnic{
	sdob:chrono::NaiveDateTime,
	patientguid:uint8,
	identifier:widechar,
	ethnicity:Ethnicity,
	gender:Gender,
	lastname:widechar,
	secondlastname:widechar,
	firstname:widechar,
	middleinitial:widechar,
	statusbits:StatusBits,
	identifierext:widechar,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Ethnicity{
	#[default]
	NONE,
	CAUCASIAN,
	ASIAN,
	BLACK,
	AMERICANINDIAN,
	HISPANIC,
	OTHER,
	ABORIGINAL,
	POLYNESIAN,
	INDIAN,
	ESKIMO,
	HAWAIIAN,
	PACIFICISLANDER,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Gender{
	#[default]
	MALE,
	FEMALE,
	UNKNOWN,
	UNDEFINED,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum StatusBits{
	#[default]
	PATIENTID_CONFIRMED,
	CONTINUOUS_MONITORED,
}

pub impl CPatientDEthnic {
	fn new() -> Self { todo!() }
}
impl Object for CPatientDEthnic{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmPATIENT }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmPATIENTSpecies::CPatientDEthnic) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmPATIENT for CPatientCStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CPatientCStd{
	barcodeenabled:boolean,
	externalport:ExternalPort,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExternalPort{
	#[default]
	NONE,
	UNKNOWN,
	PORT_1,
	PORT_2,
	PORT_3,
	PORT_4,
}

pub impl CPatientCStd {
	fn new() -> Self { todo!() }
}
impl Object for CPatientCStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmPATIENT }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmPATIENTSpecies::CPatientCStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmPATIENT for CPatientPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CPatientPSelector{
	value:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CPatientPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CPatientPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmPATIENT }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmPATIENTSpecies::CPatientPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmPATIENT for CPatientDExtStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CPatientDExtStd{
	container:Object::ANY_CLASS,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CPatientDExtStd {
	fn new() -> Self { todo!() }
}
impl Object for CPatientDExtStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmPATIENT }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATAEXT }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmPATIENTSpecies::CPatientDExtStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

