use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmRECORD:Downcast{}

#[derive(Clone,Debug)]
pub enum FmRECORDSpecies{
	SpSTANDARD,
	SpNIBP,
	SpTEMPERATURE,
	SpSPO2,
	SpHEIGHT,
	SpWEIGHT,
	SpCO2,
	SpRESTING_ECG,
	SpSTRESS_ECG,
	SpSPIROMETRY,
	SpTEMPERATURE_PROBE,
	SpBATTERY,
	SpHOURS,
	SpTEMPERATURE_DOCK,
	SpPROBE_EXPIRATION,
	SpPATIENTMOTION_BED_SENSOR_EXPIRATION,
	SpPATIENTMOTION_CHAIR_SENSOR_EXPIRATION,
	SpRECORD,
}
impl ObjectSpecies for FmRECORDSpecies{}

impl FmRECORD for CRecordDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CRecordDStd{
	recordcount:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CRecordDStd {
	fn new() -> Self { todo!() }
}
impl Object for CRecordDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmRECORD }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmRECORDSpecies::CRecordDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmRECORD for CRecordPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CRecordPSelector{
	speciesselector:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CRecordPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CRecordPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmRECORD }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmRECORDSpecies::CRecordPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmRECORD for CRecordDExtStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CRecordDExtStd{
	recordcontainer:Object::ANY_CLASS,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CRecordDExtStd {
	fn new() -> Self { todo!() }
}
impl Object for CRecordDExtStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmRECORD }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATAEXT }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmRECORDSpecies::CRecordDExtStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

