use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmRESPIRATION:Downcast{}

#[derive(Clone,Debug)]
pub enum FmRESPIRATIONSpecies{
	SpSTANDARD,
	SpDATA,
}
impl ObjectSpecies for FmRESPIRATIONSpecies{}

impl FmRESPIRATION for CRespDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CRespDStd{
	stime:chrono::NaiveDateTime,
	status:Status,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	respiration:uint8,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

pub impl CRespDStd {
	fn new() -> Self { todo!() }
}
impl Object for CRespDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmRESPIRATION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmRESPIRATIONSpecies::CRespDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmRESPIRATION for CRespDData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CRespDData{
	stime:chrono::NaiveDateTime,
	status:Status,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	respiration:Object::CNumDStd,
	confidence:Object::CNumDStd,
	breathtobreathtime:Object::CNumDStd,
	elapsedtimesincelast:Object::CNumDStd,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
	BREATH_FOUND,
	SEARCHING_FOR_BOARD,
	NO_PROBE,
	INCORRECT_PROBE,
	SEARCHING_FOR_BREATH,
	BAD_PROBE,
	PROBE_EXPIRED,
	NO_CABLE,
	INCORRECT_CABLE,
	BAD_CABLE,
	CABLE_EXPIRED,
	VALUE_NOT_ACQUIRED,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	RESPIRATION,
	CO2,
	PATIENTMOTION_BED_SENSOR,
	PATIENTMOTION_CHAIR_SENSOR,
	ECG,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
	ACOUSTIC,
	PLETH,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

pub impl CRespDData {
	fn new() -> Self { todo!() }
}
impl Object for CRespDData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmRESPIRATION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmRESPIRATIONSpecies::CRespDData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

