use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmSESSION:Downcast{}

#[derive(Clone,Debug)]
pub enum FmSESSIONSpecies{
	SpECGTEST,
	SpULTRARECORD,
	SpSPIROTEST,
	SpVITALSRECORD,
	SpMONITOR,
	SpTRENDRECORD,
	SpECGSETUP,
	SpULTRASETUP,
	SpSPIROSETUP,
	SpCP100_200_SYSSETUP,
	SpMSG_ARGUMENTS,
	SpSELECTOR,
}
impl ObjectSpecies for FmSESSIONSpecies{}

impl FmSESSION for CSESDEcgTest{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSESDEcgTest{
	testdate:chrono::NaiveDateTime,
	testguid:uint8,
	patientageattest:uint16,
	testconfirmed:boolean,
	abnormalecg:boolean,
	testprinted:boolean,
	testexported:boolean,
	deviceid:widechar,
	medicationhistory1:widechar,
	medicationhistory2:widechar,
	medicationhistory3:widechar,
	clinicalhistory1:widechar,
	clinicalhistory2:widechar,
	clinicalhistory3:widechar,
	cust1_label:widechar,
	cust2_label:widechar,
	cust1_value:widechar,
	cust2_value:widechar,
	comments:widechar,
	cpt:Object::CPatientDEthnic,
	cdevice:Object::CDeviceDStd,
	cmodulehost:Object::CModuleDStd,
	cmodulefrontend:Object::CModuleDStd,
	cmodulemeans:Object::CModuleDStd,
	cint:Object::CInterpDStd,
	cwa:Object::CColDWave,
	aecg:Object::CColDWave,
	cpa:Object::CECGDPacer,
	cmm:Object::CColDLeadMeas,
	cgm:Object::CECGDGlobalMeas,
	cbp:Object::CNIBPDStd,
	cheight:Object::CHeightDStd,
	cweight:Object::CWeightDStd,
	practiceid:widechar,
	orderid:widechar,
	patientageunit:PatientAgeUnit,
	physicianid:widechar,
	technicianid:widechar,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum PatientAgeUnit{
	#[default]
	MONTHS,
	YEARS,
	DAYS,
	HOURS,
	MINUTES,
}

pub impl CSESDEcgTest {
	fn new() -> Self { todo!() }
}
impl Object for CSESDEcgTest{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSESSION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSESSIONSpecies::CSESDEcgTest) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSESSION for CSESDUltraRec{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSESDUltraRec{
	stime:chrono::NaiveDateTime,
	deviceinfo:Object::CDeviceDStd,
	nibpdata:Object::CNIBPDStd,
	tempdata:Object::CTempDStd,
	spo2data:Object::CSpO2DStd,
	weightdata:Object::CWeightDStd,
	heightdata:Object::CHeightDStd,
	bmidata:Object::CBMIDStd,
	respdata:Object::CRespDStd,
	paindata:Object::CPainDStd,
	clinician:Object::CClinicianDStd,
	patient:Object::CPatientDStd,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CSESDUltraRec {
	fn new() -> Self { todo!() }
}
impl Object for CSESDUltraRec{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSESSION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSESSIONSpecies::CSESDUltraRec) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSESSION for CSESDSpiroTest{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSESDSpiroTest{
	testguid:uint8,
	packsperday:uint8,
	yearssmoking:uint8,
	lungagepredicted:uint8,
	patientageattest:uint16,
	normreferences:uint16,
	normresults:float,
	normlln:float,
	testprotocol:uint16,
	normdataextrapolated:boolean,
	atsinterpretationobstruction:uint16,
	atsinterpretationrestriction:uint16,
	adultnorm:uint16,
	pediatricnorm:uint16,
	besteffort:uint16,
	revformula:uint16,
	fevformula:uint16,
	fev1_p_acrossefforts:float,
	userid:widechar,
	atsreproduciblepre:boolean,
	atsreproduciblepost:boolean,
	testconfirmed:boolean,
	testprinted:boolean,
	testexported:boolean,
	prequalitygrade:widechar,
	postqualitygrade:widechar,
	comments:widechar,
	patientinfo:Object::CPatientDEthnic,
	hostdeviceinfo:Object::CDeviceDStd,
	hostsoftwaremoduleinfo:Object::CModuleDStd,
	spiroefforts:Object::CColDSpiro,
	medicationlist:Object::CColDMedsStrings,
	clinicalhistory:Object::CColDClinHistStrings,
	interpresults:Object::CColDInterpResults,
	height:Object::CHeightDStd,
	weight:Object::CWeightDStd,
	calibrationinfo:Object::CCalDSpiro,
	deviceid:widechar,
	practiceid:widechar,
	orderid:widechar,
	patientageunit:PatientAgeUnit,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum PatientAgeUnit{
	#[default]
	MONTHS,
	YEARS,
	DAYS,
	HOURS,
	MINUTES,
}

pub impl CSESDSpiroTest {
	fn new() -> Self { todo!() }
}
impl Object for CSESDSpiroTest{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSESSION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSESSIONSpecies::CSESDSpiroTest) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSESSION for CSESDVitalsRRecord{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSESDVitalsRRecord{
	testdate:chrono::NaiveDateTime,
	testguid:uint8,
	orderid:widechar,
	container:Object::ANY_CLASS,
	confirmedstate:ConfirmedState,
	trendtype:TrendType,
	customdatabits:CustomDataBits,
	morefragments:boolean,
	identifier:uint8,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ConfirmedState{
	#[default]
	NONE,
	UNCONFIRMED,
	CONFIRMED,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum TrendType{
	#[default]
	NONE,
	UNKNOWN,
	TIMED_INTERVAL,
	PHYSIOLOGIC_ALARM,
	EPISODIC_READING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum CustomDataBits{
	#[default]
	CUSTOM_DATA_ENABLED,
	DEPRECATE_STANDARD_MODIFIER,
}

pub impl CSESDVitalsRRecord {
	fn new() -> Self { todo!() }
}
impl Object for CSESDVitalsRRecord{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSESSION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSESSIONSpecies::CSESDVitalsRRecord) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSESSION for CSESDMonitor{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSESDMonitor{
	identifier:uint32,
	container:Object::ANY_CLASS,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CSESDMonitor {
	fn new() -> Self { todo!() }
}
impl Object for CSESDMonitor{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSESSION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSESSIONSpecies::CSESDMonitor) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSESSION for CSESCEcgSetup{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSESCEcgSetup{
	crhythmreport:Object::CECGCRhythmReport,
	cmiscsettings:Object::CECGCMiscellaneous,
	cautoreport1:Object::CECGCAutoReport,
	cautoreport2:Object::CECGCAutoReport,
	cautoreport3:Object::CECGCAutoReport,
	cautoreport4:Object::CECGCAutoReport,
	autoreportnum:uint16,
	autorpt1printer:uint16,
	autorpt2printer:uint16,
	cautoreport5:Object::CECGCAutoReport,
	cautoreport6:Object::CECGCAutoReport,
	cautoreport7:Object::CECGCAutoReport,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CSESCEcgSetup {
	fn new() -> Self { todo!() }
}
impl Object for CSESCEcgSetup{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSESSION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSESSIONSpecies::CSESCEcgSetup) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSESSION for CSESCSpiroSetup{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSESCSpiroSetup{
	hostdeviceinfo:Object::CDeviceDStd,
	hostsoftwaremoduleinfo:Object::CModuleDStd,
	spirosettings:Object::CSpiroCStd,
	interplist:Object::CColDInterpStrings,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CSESCSpiroSetup {
	fn new() -> Self { todo!() }
}
impl Object for CSESCSpiroSetup{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSESSION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSESSIONSpecies::CSESCSpiroSetup) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSESSION for CSESCCP100_200SysSetup{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSESCCP100_200SysSetup{
	cmodulearray:Object::CColDModule,
	cuserarray:Object::CColDUser,
	cfax:Object::CFaxCStd,
	cdevice:Object::CDeviceCStd,
	cprinter:Object::CPrinterCStd,
	cheight:Object::CHeightCStd,
	cweight:Object::CWeightCStd,
	cfaxconfig:Object::CModemCStd,
	cfaaconfig:Object::CModemCStd,
	cethernet:Object::CNetworkCEth,
	cwifi:Object::CNetworkCWIFI,
	cworkstation:Object::CNetworkCWkst,
	cuiconfig:Object::CUICStd,
	cuidisplaysettings:Object::CUICDisplayUnits,
	ccontactlist:Object::CColDContactInfo,
	ckeylock:Object::CUICKeyLock,
	clcd:Object::CDeviceCLCD,
	dcpenabled:boolean,
	dcpport:uint32,
	sntpaddress:Object::CNetworkCAddress,
	cdevctime:Object::CDeviceCTime,
	cdevcsvcmon:Object::CDeviceCServiceMonitor,
	cwireless:Object::CNetworkCEth,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CSESCCP100_200SysSetup {
	fn new() -> Self { todo!() }
}
impl Object for CSESCCP100_200SysSetup{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSESSION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSESSIONSpecies::CSESCCP100_200SysSetup) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSESSION for CSESPMsgArguments{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSESPMsgArguments{
	speciesselector:uint32,
	ioparametera:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CSESPMsgArguments {
	fn new() -> Self { todo!() }
}
impl Object for CSESPMsgArguments{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSESSION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSESSIONSpecies::CSESPMsgArguments) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSESSION for CSESPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSESPSelector{
	speciesselector:uint32,
	objectselector:uint32,
	interval:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CSESPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CSESPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSESSION }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSESSIONSpecies::CSESPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

