use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmSPHB:Downcast{}

#[derive(Clone,Debug)]
pub enum FmSPHBSpecies{
	SpDATA,
	SpSTANDARD,
	SpSELECTOR,
}
impl ObjectSpecies for FmSPHBSpecies{}

impl FmSPHB for CSpHBDData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSpHBDData{
	stime:chrono::NaiveDateTime,
	status:Status,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	thb:Object::CNumDStd,
	thbmmol_l:Object::CNumDStd,
	confidence:Object::CNumDStd,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	PULSE_FOUND,
	SEARCHING_FOR_BOARD,
	NO_PROBE,
	INCORRECT_PROBE,
	SEARCHING_FOR_PULSE,
	BAD_PROBE,
	PROBE_EXPIRED,
	NO_CABLE,
	INCORRECT_CABLE,
	BAD_CABLE,
	CABLE_EXPIRED,
	VALUE_NOT_ACQUIRED,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	MASIMO_MX,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
	ARTERIAL,
	VENOUS,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

pub impl CSpHBDData {
	fn new() -> Self { todo!() }
}
impl Object for CSpHBDData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSPHB }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSPHBSpecies::CSpHBDData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSPHB for CSpHBCStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSpHBCStd{
	displayunits:DisplayUnits,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum DisplayUnits{
	#[default]
	UNITS_GDL,
	UNITS_MMOLL,
}

pub impl CSpHBCStd {
	fn new() -> Self { todo!() }
}
impl Object for CSpHBCStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSPHB }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSPHBSpecies::CSpHBCStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSPHB for CSpHBCData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSpHBCData{
	thbexponent:int8,
	averagingmode:AveragingMode,
	mode:Mode,
	precisionmode:PrecisionMode,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum AveragingMode{
	#[default]
	NONE,
	SHORT,
	MEDIUM,
	LONG,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
	ARTERIAL,
	VENOUS,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum PrecisionMode{
	#[default]
	NONE,
	UNKNOWN,
	NEAREST_TENTH,
	NEAREST_HALF,
	WHOLE,
}

pub impl CSpHBCData {
	fn new() -> Self { todo!() }
}
impl Object for CSpHBCData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSPHB }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSPHBSpecies::CSpHBCData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSPHB for CSpHBPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSpHBPSelector{
	value:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CSpHBPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CSpHBPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSPHB }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSPHBSpecies::CSpHBPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

