use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmSPIRO:Downcast{}

#[derive(Clone,Debug)]
pub enum FmSPIROSpecies{
	SpSTANDARD,
}
impl ObjectSpecies for FmSPIROSpecies{}

impl FmSPIRO for CSpiroDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSpiroDStd{
	effortdatetime:chrono::NaiveDateTime,
	efforttype:uint16,
	effortnumber:uint8,
	parameters:float,
	posttest:boolean,
	atsacceptable:boolean,
	rank:uint8,
	qualitymessage:QualityMessage,
	flowdata:Object::CWaveDStd,
	volumedata:Object::CWaveDStd,
	dataconfig:Object::CWaveCStd,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum QualityMessage{
	#[default]
	DONT_HESITATE,
	BLAST_OUT_FASTER,
	BLAST_OUT_LONGER,
	BLAST_OUT_HARDER,
	DEEPER_BREATH,
	GOOD_EFFORT,
}

pub impl CSpiroDStd {
	fn new() -> Self { todo!() }
}
impl Object for CSpiroDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSPIRO }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSPIROSpecies::CSpiroDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSPIRO for CSpiroCStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSpiroCStd{
	adultnorm:uint16,
	pediatricnorm:uint16,
	besteffort:uint16,
	revformula:uint16,
	fevformula:uint16,
	enablepredpoints:boolean,
	enablepredcurves:boolean,
	enableatsinterpretation:boolean,
	enablecompositenorms:boolean,
	caltransducerlot:uint16,
	caltransducercode:ansichar,
	calmaxvariation:uint8,
	calenableautoreport:boolean,
	screendefaultcurve:uint16,
	screendisplayparams:uint32,
	printtests:uint16,
	printcurves:uint16,
	printparams:uint32,
	printscale:uint16,
	printatsinterp:boolean,
	printlungage:boolean,
	printunconfirmedreport:boolean,
	printreviewedby:boolean,
	printpatientcoop:boolean,
	printpatienteducation:boolean,
	printenableauto:boolean,
	commautosend:uint16,
	patientfirstnameenabled:boolean,
	patientsecondlastnameenabled:boolean,
	patientmiddleinitialenabled:boolean,
	patientusedob:boolean,
	patientweightenabled:boolean,
	patientmedicationenabled:boolean,
	patienthistoryenabled:boolean,
	patientcommentsenabled:boolean,
	testprotocol:TestProtocol,
	enableautomaticqualitycheck:boolean,
	printqualitygrades:boolean,
	patientsmokeyearsenabled:boolean,
	patientpacksperdayenabled:boolean,
	printphysicianscomments:boolean,
	printerselection:uint8,
	incentiveoption:uint8,
	printatsrepronotmet:boolean,
	enabledailyreminder:boolean,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum TestProtocol{
	#[default]
	NONE,
	PCP,
	NIOSH,
	OSHA,
	SSD,
}

pub impl CSpiroCStd {
	fn new() -> Self { todo!() }
}
impl Object for CSpiroCStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSPIRO }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSPIROSpecies::CSpiroCStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

