use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmSPMET:Downcast{}

#[derive(Clone,Debug)]
pub enum FmSPMETSpecies{
	SpDATA,
	SpSELECTOR,
}
impl ObjectSpecies for FmSPMETSpecies{}

impl FmSPMET for CSpMETDData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSpMETDData{
	stime:chrono::NaiveDateTime,
	status:Status,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	spmet:Object::CNumDStd,
	confidence:Object::CNumDStd,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	PULSE_FOUND,
	SEARCHING_FOR_BOARD,
	NO_PROBE,
	INCORRECT_PROBE,
	SEARCHING_FOR_PULSE,
	BAD_PROBE,
	PROBE_EXPIRED,
	NO_CABLE,
	INCORRECT_CABLE,
	BAD_CABLE,
	CABLE_EXPIRED,
	VALUE_NOT_ACQUIRED,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	MASIMO_MX,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

pub impl CSpMETDData {
	fn new() -> Self { todo!() }
}
impl Object for CSpMETDData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSPMET }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSPMETSpecies::CSpMETDData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSPMET for CSpMETCData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSpMETCData{
	spmetexponent:int8,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CSpMETCData {
	fn new() -> Self { todo!() }
}
impl Object for CSpMETCData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSPMET }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSPMETSpecies::CSpMETCData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSPMET for CSpMETPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSpMETPSelector{
	value:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CSpMETPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CSpMETPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSPMET }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSPMETSpecies::CSpMETPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

