use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmSPOC:Downcast{}

#[derive(Clone,Debug)]
pub enum FmSPOCSpecies{
	SpDATA,
	SpSELECTOR,
}
impl ObjectSpecies for FmSPOCSpecies{}

impl FmSPOC for CSpOCDData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSpOCDData{
	stime:chrono::NaiveDateTime,
	status:Status,
	extstatus:ExtStatus,
	source:Source,
	mode:Mode,
	method:Method,
	spoc:Object::CNumDStd,
	confidence:Object::CNumDStd,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	PULSE_FOUND,
	SEARCHING_FOR_BOARD,
	NO_PROBE,
	INCORRECT_PROBE,
	SEARCHING_FOR_PULSE,
	BAD_PROBE,
	PROBE_EXPIRED,
	NO_CABLE,
	INCORRECT_CABLE,
	BAD_CABLE,
	CABLE_EXPIRED,
	VALUE_NOT_ACQUIRED,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	MASIMO_MX,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

pub impl CSpOCDData {
	fn new() -> Self { todo!() }
}
impl Object for CSpOCDData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSPOC }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSPOCSpecies::CSpOCDData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSPOC for CSpOCCData{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSpOCCData{
	spocexponent:int8,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CSpOCCData {
	fn new() -> Self { todo!() }
}
impl Object for CSpOCCData{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSPOC }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSPOCSpecies::CSpOCCData) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmSPOC for CSpOCPSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CSpOCPSelector{
	value:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CSpOCPSelector {
	fn new() -> Self { todo!() }
}
impl Object for CSpOCPSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmSPOC }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmSPOCSpecies::CSpOCPSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

