use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmTYMP:Downcast{}

#[derive(Clone,Debug)]
pub enum FmTYMPSpecies{
	SpSTANDARD,
	SpCALIBRATION,
	SpBARCODE,
	SpMSG_ARGUMENTS,
	SpTYMP_DATA,
	SpSPEAKER,
	SpMICROPHONE,
	SpLUT,
	SpTONE,
	SpADMITTANCE,
	SpPRESSURE,
	SpCOLDFIRE,
	SpSLAVE,
	SpENGINEERING,
	SpDATA_LOG,
}
impl ObjectSpecies for FmTYMPSpecies{}

impl FmTYMP for CTympDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CTympDStd{
	frequency:uint16,
	ear:Ear,
	tympresult:TympResult,
	calstatus:uint8,
	compblval:uint8,
	comptype:CompType,
	vea:uint8,
	peakya:uint8,
	tpp:uint16,
	gr:uint16,
	ya:int8,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Ear{
	#[default]
	NONE,
	LEFT,
	RIGHT,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum TympResult{
	#[default]
	INCOMPLETE,
	PASS,
	REFER,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum CompType{
	#[default]
	NONE,
	UNCOMPENSATED,
	POSITIVE_TAIL,
	NEGATIVE_TAIL,
}

pub impl CTympDStd {
	fn new() -> Self { todo!() }
}
impl Object for CTympDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmTYMP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmTYMPSpecies::CTympDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmTYMP for CTympDCal{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CTympDCal{
	status:uint8,
	date:chrono::NaiveDateTime,
	ate_version:ansichar,
	ate_ident:ansichar,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CTympDCal {
	fn new() -> Self { todo!() }
}
impl Object for CTympDCal{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmTYMP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmTYMPSpecies::CTympDCal) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmTYMP for CTympDBarCode{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CTympDBarCode{
	mcu:ansichar,
	ps:ansichar,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CTympDBarCode {
	fn new() -> Self { todo!() }
}
impl Object for CTympDBarCode{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmTYMP }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmTYMPSpecies::CTympDBarCode) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

