use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmUI:Downcast{}

#[derive(Clone,Debug)]
pub enum FmUISpecies{
	SpKEYBOARD,
	SpLOCATION,
	SpCALIBRATION,
	SpSTANDARD,
	SpDISPLAYUNITS,
	SpKEYLOCK,
}
impl ObjectSpecies for FmUISpecies{}

impl FmUI for CUIDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CUIDStd{
	keypress:uint16,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CUIDStd {
	fn new() -> Self { todo!() }
}
impl Object for CUIDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmUI }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmUISpecies::CUIDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmUI for CUIDLocation{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CUIDLocation{
	x_coordinate:uint32,
	y_coordinate:uint32,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CUIDLocation {
	fn new() -> Self { todo!() }
}
impl Object for CUIDLocation{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmUI }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmUISpecies::CUIDLocation) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmUI for CUIDCalibration{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CUIDCalibration{
	status:Status,
	uicontainer:Object::ANY_CLASS,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

pub impl CUIDCalibration {
	fn new() -> Self { todo!() }
}
impl Object for CUIDCalibration{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmUI }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmUISpecies::CUIDCalibration) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmUI for CUICStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CUICStd{
	enableuievents:boolean,
	loginrequired:boolean,
	auditinfoenabled:boolean,
	enablelockout:boolean,
	lockoutinterval:uint16,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CUICStd {
	fn new() -> Self { todo!() }
}
impl Object for CUICStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmUI }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmUISpecies::CUICStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmUI for CUICDisplayUnits{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CUICDisplayUnits{
	flowunits:FlowUnits,
	pressureunits:PressureUnits,
	temperatureunits:TemperatureUnits,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum FlowUnits{
	#[default]
	FLOW_LITER_PER_SECOND,
	FLOW_LITER_PER_MINUTE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum PressureUnits{
	#[default]
	PRESSURE_MBAR,
	PRESSURE_MMHG,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum TemperatureUnits{
	#[default]
	TEMPERATURE_CELSIUS,
	TEMPERATURE_FARENHEIT,
}

pub impl CUICDisplayUnits {
	fn new() -> Self { todo!() }
}
impl Object for CUICDisplayUnits{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmUI }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmUISpecies::CUICDisplayUnits) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmUI for CUICKeyLock{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CUICKeyLock{
	capslock:boolean,
	numlock:boolean,
	scrolllock:boolean,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CUICKeyLock {
	fn new() -> Self { todo!() }
}
impl Object for CUICKeyLock{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmUI }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmUISpecies::CUICKeyLock) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

