use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmWAVESAMPLE:Downcast{}

#[derive(Clone,Debug)]
pub enum FmWAVESAMPLESpecies{
	SpSTANDARD,
	SpECGWAVSMP,
	SpAUDIOWAVSMP,
	SpECGIMPEDWAVSMP,
	SpRWAVDETECT,
	SpPACEWAVDETECT,
	SpSPIROWAVSMP,
	SpECG_I,
	SpECG_II,
	SpECG_III,
	SpECG_AVL,
	SpECG_AVR,
	SpECG_AVF,
	SpECG_V1,
	SpECG_V2,
	SpECG_V3,
	SpECG_V4,
	SpECG_V5,
	SpECG_V6,
	SpECG_V7,
	SpECG_V3R,
	SpECG_V5R,
	SpECG_X,
	SpECG_Y,
	SpECG_Z,
	SpECG_V4R,
	SpECG_V6R,
	SpECG_V8,
	SpECG_V9,
	SpSPIROFLOW,
	SpSPIROVOLUME,
	SpPADS,
	SpRESP,
	SpOSCILLIMETRIC,
	SpMANOMETER,
	SpSPO2_SAT,
	SpSPO2_PLETH,
	SpSIQ,
	SpCO2,
	SpIBP1,
	SpIBP2,
	SpIBP3,
	SpIBP4,
	SpTEMP1,
	SpTEMP2,
	SpAUDIO,
	SpSPO2_PULSEAMPLITUDE,
	SpECG_ELECTRODE_RL,
	SpECG_ELECTRODE_LL,
	SpECG_ELECTRODE_RA,
	SpECG_ELECTRODE_LA,
	SpECG_ELECTRODE_V1,
	SpECG_ELECTRODE_V2,
	SpECG_ELECTRODE_V3,
	SpECG_ELECTRODE_V4,
	SpECG_ELECTRODE_V5,
	SpECG_ELECTRODE_V6,
	SpALL,
	SpECGWAVCFG,
	SpAUDIOWAVCFG,
	SpECGIMPEDWAVCFG,
	SpSPIROWAVECFG,
	SpRWAVDETCFG,
	SpPWAVDETCFG,
	SpSELECTOR,
	SpCONFIGURATION,
	SpDATA,
}
impl ObjectSpecies for FmWAVESAMPLESpecies{}

impl FmWAVESAMPLE for CWaveSampleDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CWaveSampleDStd{
	sequencenumber:uint32,
	sample:uint8,
	quality:Quality,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Quality{
	#[default]
	NONE,
	INVALID,
	VALID,
	DUBIOUS,
}

pub impl CWaveSampleDStd {
	fn new() -> Self { todo!() }
}
impl Object for CWaveSampleDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmWAVESAMPLE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmWAVESAMPLESpecies::CWaveSampleDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmWAVESAMPLE for CWaveSampleCStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CWaveSampleCStd{
	isactive:boolean,
	samplefrequency:int16,
	samplesize:byte,
	samplecount:int16,
	channels:byte,
	channelsequence:byte,
	annotationenabled:boolean,
	samplefrequencyexponent:int8,
	centerpoint:uint32,
	max:uint32,
	units:Units,
	unitsexponent:int8,
	resolution:uint32,
	resolutionexponent:int8,
	sampleperiod:uint32,
	encoding:uint8,
	changeattribute:ChangeAttribute,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Units{
	#[default]
	NONE,
	METERS_OF_MERCURY,
	VOLTS,
	PASCAL,
	DEGREE_CELSIUS,
	DEGREE_FAHRENHEIT,
	DEGREE_KELVIN,
	PERCENT,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ChangeAttribute{
	#[default]
	IsActive_Member,
	SampleFrequency_Member,
	SampleSize_Member,
	SampleCount_Member,
	Channels_Member,
	ChannelSequence_Member,
	AnnotationEnabled_Member,
	SampleFrequencyExponent_Member,
	CenterPoint_Member,
	Max_Member,
	Units_Member,
	UnitsExponent_Member,
	Resolution_Member,
	ResolutionExponent_Member,
	SamplePeriod_Member,
	Encoding_Member,
	ChangeAttribute_Member,
}

pub impl CWaveSampleCStd {
	fn new() -> Self { todo!() }
}
impl Object for CWaveSampleCStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmWAVESAMPLE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmWAVESAMPLESpecies::CWaveSampleCStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmWAVESAMPLE for CWaveSampleCRPDetect{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CWaveSampleCRPDetect{
	isactive:boolean,
	detectchannel:byte,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CWaveSampleCRPDetect {
	fn new() -> Self { todo!() }
}
impl Object for CWaveSampleCRPDetect{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmWAVESAMPLE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmWAVESAMPLESpecies::CWaveSampleCRPDetect) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmWAVESAMPLE for CWaveSamplePSelector{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CWaveSamplePSelector{
	selector:uint8,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CWaveSamplePSelector {
	fn new() -> Self { todo!() }
}
impl Object for CWaveSamplePSelector{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmWAVESAMPLE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnPARAMETER }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmWAVESAMPLESpecies::CWaveSamplePSelector) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmWAVESAMPLE for CWaveSampleAStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CWaveSampleAStd{
	sequencenumber:uint32,
	sampleindex:uint16,
	annotation:Annotation,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Annotation{
	#[default]
	NONE,
	PACER,
	QRS,
	SYNC,
	DEFIB_FIRED,
	LEAD_OFF,
	BREATH_DETECT,
	END_BREATH_DETECT,
	PULSE_DETECT,
	PULSE_TONE,
	VFIB,
	ASYSTOLE,
	VTACH,
	PVC_RUN,
	VRHYTHM,
	COUPLET,
	PVC_MIN,
	BIGEMINY,
	TRIGEMINY,
	TACHYCARDIA,
	BRADYCARDIA,
	PAUSE,
	MISSED_BEAT,
	MULTIFOCAL,
	R_ON_T,
	PACER_NON_CAPTURE,
	PACER_NON_FIRE,
	VSYNC,
	APNEA_INDEX_TOTAL,
	APNEA_10_20,
	APNEA_20_29,
	APNEA_30,
	DESAT_COUNT,
	LEAD_ON,
	ELECTRODE_ON,
	ELECTRODE_OFF,
}

pub impl CWaveSampleAStd {
	fn new() -> Self { todo!() }
}
impl Object for CWaveSampleAStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmWAVESAMPLE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnANNOTATION }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmWAVESAMPLESpecies::CWaveSampleAStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmWAVESAMPLE for CWaveSampleDExtStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CWaveSampleDExtStd{
	container:Object::ANY_CLASS,
	#[getter(skip)]
	raw_object:RawObject,
}

pub impl CWaveSampleDExtStd {
	fn new() -> Self { todo!() }
}
impl Object for CWaveSampleDExtStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmWAVESAMPLE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATAEXT }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmWAVESAMPLESpecies::CWaveSampleDExtStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

