use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmWAVE:Downcast{}

#[derive(Clone,Debug)]
pub enum FmWAVESpecies{
	SpSTANDARD,
	SpECGWAVE,
	SpAUDIOWAVE,
	SpLEADIMPEDWAVE,
	SpSPIROWAVE,
	SpECGWAVCFG,
	SpAUDIOWAVCFG,
	SpECGLEADIMPEDWAVCFG,
	SpSPIROWAVCFG,
}
impl ObjectSpecies for FmWAVESpecies{}

impl FmWAVE for CWaveDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CWaveDStd{
	waveidentifier:WaveIdentifier,
	quality:byte,
	sample:int16,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum WaveIdentifier{
	#[default]
	I,
	II,
	III,
	AVR,
	AVL,
	AVF,
	V1,
	V2,
	V3,
	V4,
	V5,
	V6,
	V7,
	V3R,
	V5R,
	X,
	Y,
	Z,
	V4R,
	V6R,
	V8,
	V9,
	SpiroFlow,
	SpiroVolume,
}

pub impl CWaveDStd {
	fn new() -> Self { todo!() }
}
impl Object for CWaveDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmWAVE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmWAVESpecies::CWaveDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmWAVE for CWaveCStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CWaveCStd{
	isactive:boolean,
	powerfreq:PowerFreq,
	samplefrequency:int16,
	samplesize:byte,
	samplecount:int16,
	resolution:int32,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum PowerFreq{
	#[default]
	NONE,
	FIFTY,
	SIXTY,
}

pub impl CWaveCStd {
	fn new() -> Self { todo!() }
}
impl Object for CWaveCStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmWAVE }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmWAVESpecies::CWaveCStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

