use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait FmWEIGHT:Downcast{}

#[derive(Clone,Debug)]
pub enum FmWEIGHTSpecies{
	SpSTANDARD,
}
impl ObjectSpecies for FmWEIGHTSpecies{}

impl FmWEIGHT for CWeightDStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CWeightDStd{
	stime:chrono::NaiveDateTime,
	status:Status,
	weight:uint32,
	stabilization:boolean,
	extstatus:ExtStatus,
	mode:Mode,
	method:Method,
	source:Source,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
	#[default]
	DATA_NOT_AVAIL,
	DATA_COMPLETE,
	DATA_FAILURE,
	DATA_EXCEPTION,
	DATA_PROCESSING,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExtStatus{
	#[default]
	NONE,
	OVER_CAPACITY,
	ANALOG_ERROR,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Mode{
	#[default]
	NONE,
	UNKNOWN,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Method{
	#[default]
	NONE,
	UNKNOWN,
	MANUAL,
	DEVICE,
	REMOTE,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
	#[default]
	NONE,
	UNKNOWN,
	TANITA,
	PELOUZE,
}

pub impl CWeightDStd {
	fn new() -> Self { todo!() }
}
impl Object for CWeightDStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmWEIGHT }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnDATA }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmWEIGHTSpecies::CWeightDStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl FmWEIGHT for CWeightCStd{}
#[derive(Getters,Builder,Clone,Debug)]
pub struct CWeightCStd{
	enabled:boolean,
	manualweight:boolean,
	weightdisplayunits:WeightDisplayUnits,
	weightdefault:uint32,
	externalport:ExternalPort,
	#[getter(skip)]
	raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum WeightDisplayUnits{
	#[default]
	UNITS_LBS,
	UNITS_KG,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExternalPort{
	#[default]
	NONE,
	UNKNOWN,
	PORT_1,
	PORT_2,
	PORT_3,
	PORT_4,
}

pub impl CWeightCStd {
	fn new() -> Self { todo!() }
}
impl Object for CWeightCStd{
	fn get_family(&self) -> datatypes::Family { datatypes::Family::FmWEIGHT }
	fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::GnCONFIG }
	fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FmWEIGHTSpecies::CWeightCStd) }
	fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

