use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum BatterySpecies{
    Standard,   //0, data
    Service,    //1, data
    Select      //1, parameter
}
impl ObjectSpecies for BatterySpecies{}

#[derive(Getters,Builder,Clone)]
pub struct Battery{
    voltage:i16,                //Voltage in mV
    current:u16,                //Current in mA
    level:u16,                  //Charge level
    status:Status,              //See enum
    ac_connected:bool,          //True when AC is connected
    chemistry:Option<Chemistry>,//see enum
    percentage:u16,             //Current battery charge percentage (1-100)
    full_charge_time:u16,       //Average time to full charge (min)
    life_remaining:u16,         //% of battery life remaining (1-100)
    #[getter(skip)]
    raw_object:RawObject
}

#[derive(Clone,Default,Debug,Copy)]
pub enum Status{
    #[default]
    NotCharging,    //0
    FastCharge,     //1
    TrickleCharge,  //2
    NoBattery,      //3
}

#[derive(Clone,Default,Debug,Copy)]
pub enum Chemistry{
    #[default]
    None,       //0
    Unknown,    //1
    Alkaline,   //2
    NiMH,       //3
    LiIon,      //4
}

impl Object for Battery{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Battery }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(BatterySpecies::Standard) }
    fn get_raw_object(&self) -> super::RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct BattService{
    identifier:u32,
    data:u8,
    #[getter(skip)]
    raw_object:RawObject
}

impl Object for BattService{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Battery }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(BatterySpecies::Service) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct Select{
    select:u32,
    io_param_a:u32,
    #[getter(skip)]
    raw_object:RawObject
}

impl Object for Select{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Battery }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(BatterySpecies::Select) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
