use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use chrono;
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum BMISpecies{
    Data,   //0, data
    Config, //0, config
}
impl ObjectSpecies for BMISpecies{}

#[derive(Getters,Builder,Clone)]
pub struct BMIData{
    time:chrono::NaiveDateTime,
    status:Status,
    weight:u32,
    height:u32,
    bmi:u32,
    extended_status:u16,    //Defaults to 0
    source:Source,             
    mode:Mode,               
    method:Method,            
    bmi_extended:f32,       //Just a float version of BMI
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum Status{
    #[default]
    DataNotAvail,   //0
    DataComplete,   //1
    DataFail,       //2
    DataException,  //3
    DataProcessing, //4
}

#[derive(Clone,Default,Debug)]
pub enum Source{
    #[default]
    None,           //0
    Unknown,        //1
    Computed        //10
}

#[derive(Clone,Default,Debug)]
pub enum Mode{
    #[default]
    None,           //0
    Unknown,        //1
}

#[derive(Clone,Default,Debug)]
pub enum Method{
    #[default]
    None,           //0
    Unknown,        //1
    Manual,         //2
    Device,         //3
    Remote,         //4
}

impl Object for BMIData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::BodyMassIndex }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(BMISpecies::Data) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct BMIConfig{
    enabled:bool,
    weight_units:WeightUnits,
    height_units:HeightUnits,
    #[getter(skip)]
    raw_object:RawObject
}

#[derive(Clone,Default,Debug)]
pub enum WeightUnits{
    #[default]
    Pounds,         //0
    Kilogrammes,    //1
}

#[derive(Clone,Default,Debug)]
pub enum HeightUnits{
    #[default]
    Inches,         //0
    Centimetres,    //1
    FeetInches,     //2
}

impl Object for BMIConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::BodyMassIndex }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(BMISpecies::Config) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
