use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use chrono;
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum ClinicianSpecies{
    Data,   //0, data
    Config, //0, config
}
impl ObjectSpecies for ClinicianSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct Clinician{
    time:chrono::NaiveDateTime,
    status:Status,
    extended_status:ExtendedStatus,
    source:Source,
    mode:Mode,
    method:Method,
    identifier:String,
    extended_identifier:String,
    last_name:String,
    first_name:String,
    middle_name:String,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Default,Debug,Clone)]
pub enum Status{
    #[default]
    DataNotAvail,   //0
    DataComplete,   //1
    DataFailure,    //2
    DataException,  //3
    DataProcessing, //4
}

#[derive(Default,Debug,Clone)]
pub enum ExtendedStatus{
    #[default]
    None            //0
}

#[derive(Default,Debug,Clone)]
pub enum Source{
    #[default]
    None,           //0
    Unknown,        //1
    Barcode         //10
}

#[derive(Default,Debug,Clone)]
pub enum Mode{
    #[default]
    None,           //0
    Unknown,        //1
}

#[derive(Default,Debug,Clone)]
pub enum Method{
    #[default]
    None,           //0
    Unknown,        //1
    Manual,         //2
    Device,         //3
    Remote,         //4
}

impl Object for Clinician{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Clinician }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ClinicianSpecies::Data) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct Config{
    barcode_enabled:bool,
    external_port:Port,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Default,Debug,Clone)]
pub enum Port{
    #[default]
    None,       //0
    Unknown,    //1
    Port1,      //10
    Port2,      //11
    Port3,      //12
    Port4,      //13
}

impl Object for Config{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Clinician }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ClinicianSpecies::Config) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
