use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

//*******************
//CHECK DOCUMENTATION
//*******************
#[derive(Clone,Debug,Copy)]
pub enum DataTransferSpecies{
    Data,
    Description,
}
impl ObjectSpecies for DataTransferSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct DataTransferExtended{
    sequence_number:u32,
    identifier:u32,
    data:Vec<u8>,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for DataTransferExtended{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::DataTransfer }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::DataExt }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DataTransferSpecies::Data) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct DataTransferConfigDesc{
    init_sequence_number:u32,
    total_data_size:u32,
    packet_size:u16,
    role:Role,
    identifier:u32,
    descriptor:&'static dyn Object,
    window_size:u16,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Default,Debug,Copy,Clone)]
pub enum Role{
    #[default]
    None,
    DataRecipient,
    DataSource,
}

impl Object for DataTransferConfigDesc{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::DataTransfer }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DataTransferSpecies::Description) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
