use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum ErrorSpecies{
    Error,
    ErrorSelect,
    ExtendedError,
    ErrorDescription,
}
impl ObjectSpecies for ErrorSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct Error{
    error_id:u32,
    error_code:&'static dyn ErrorCode, //from u16
    description:String,
    returned_data:Vec<u8>,
    #[getter(skip)]
    species:ErrSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

pub trait ErrorCode{}
impl ErrorCode for CommonErrors{}
impl ErrorCode for NIBPError{}
impl ErrorCode for SPO2Error{}
impl ErrorCode for HRPRError{}
impl ErrorCode for SPHBError{}
impl ErrorCode for SPCOError{}
impl ErrorCode for SPOCError{}
impl ErrorCode for SPMETError{}
impl ErrorCode for RespirationError{}
impl ErrorCode for TempError{}
impl ErrorCode for DisplayError{}
impl ErrorCode for BatteryError{}
impl ErrorCode for PrinterError{}
impl ErrorCode for ScaleError{}
impl ErrorCode for PatientMotionError{}
impl ErrorCode for SecurityProviderError{}
impl ErrorCode for ECGError{}
impl ErrorCode for UpdateHostEndError{}
impl ErrorCode for UpdateDeviceEndError{}

#[derive(Debug,Default,Clone)]
pub enum CommonErrors{
    #[default]
    None,                               //0x00
    POSTFPROM,                          //0x01
    POSTSRAM,                           //0x02
    POSTFactory,                        //0x03
    POSTUser,                           //0x04
    POSTADC,                            //0x05
    POSTSensor,                         //0x06
    POSTPrinter,                        //0x07
    POSTRTC,                            //0x08
    POSTNoCal,                          //0x09
    POSTCalBad,                         //0x0a
    POSTValve,                          //0x0b
    POSTLog,                            //0x0c
    POSTEEPatientData,                  //0x0d
    POSTMax,                            //0x0e
    POSTErrorCheck,                     //0x0f
    POSTNotReady,                       //0x10
    POSTATEFail,                        //0x11
    InvalidAlgorithm,                   //0x12
    InvalidInitCode,                    //0x13
    InvalidPMode,                       //0x14
    InvalidConfig,                      //0x15
    CycMalfunction,                     //0x16
    CycTempF,                           //0x17
    VoltageBatteryLow,                  //0x18
    VoltageBatteryHigh,                 //0x19
    VoltageBadVibes,                    //0x1a
    PathNotFound,                       //0x1b
    AccessDenied,                       //0x1c
    FileDeleted,                        //0x1d
    FileExpired,                        //0x1e
    WriteFailed,                        //0x1f
    EraseFailed,                        //0x20
    CableLifeNearExpiration,            //0x21
    CableExpired,                       //0x22
    ProbeLifeNearExpiration,            //0x23
    ProbeExpired,                       //0x24
    DemoMode,                           //0x25
    PatientIDUnconfirmed,               //0x26
    LocationIDUnconfirmed,              //0x27
    LicenseUnavailable,                 //0x28
    InsufficientSensors,                //0x29
    PatientIDUnknown,                   //0x2a
    WiredDropOut,                       //0x2b
    StandbyMode,                        //0x2c
    PauseMode,                          //0x2d
    DropOutBatteryLow,                  //0x2e
    USBCommunicationFailure,            //0x2f
    CommsModuleFailure,                 //0x30
    WirelessDropOut,                    //0x31
    PatientIDLocationIDUnconfirmed,     //0x32
    FixedLocationConflict,              //0x33
    PatientNodAdmitted,                 //0x34
    PatientNotMeasured,                 //0x35
    PatientTrendDataIssue,              //0x36
    UnknownParameter,                   //0x37
    IncompatibleModule,                 //0x38
    AlarmGatewayLostConnection,         //0x39
    IncompatibleCustomDataModule,       //0x3a
    NotReady,                           //0x3b
    UpdateFileTransferTerminated,       //0x3c
    UpdateFileCorrupted,                //0x3d
    UpdateBadUpdateToken,               //0x3e
    UpdateFileSystemOutOfSpace,         //0x3f
    POSTAccelerometer,                  //0x40
    POSTLEDDriver,                      //0x41
    UpdateSubsystemInstallFailure,      //0x42
    ConfigurationInstallFailure,        //0x43
    Timeout,                            //0x44
    PatientIDMissing,                   //0x45
    ClinicianIDMissing,                 //0x46
    DateTimeMismatch,                   //0x47
    UpdateInternalError,                //0x48
    UpdateManagerInternalError,         //0x49
}

#[derive(Debug,Default,Clone)]
pub enum NIBPError{
    #[default]
    None,                           //0x00
    POSTSafetyCalLoadFail,          //0x01
    POSTSafetyRomChecksumFailed,    //0x02
    POSTSafetyNotCalibrated,        //0x03
    CycOverPressure,                //0x04
    CycTooSoon,                     //0x05
    Cyc9mm,                         //0x06
    CantClearFailSafe,              //0x07
    SafetyProcNoResponse,           //0x08
    SafetyStatModeTooLong,          //0x09
    SafetyXDCRFail,                 //0x0a
    ComBadChecksum,                 //0x0b
    ComBadCommand,                  //0x0c
    ComBadData,                     //0x0d
    ComRange,                       //0x0e
    ComNoPOST,                      //0x0f
    ComNoClear,                     //0x10
    ComNotCommand,                  //0x11
    ComTimeout,                     //0x12
    ComWrongResponseHeader,         //0x13
    ComResponseBadChecksum,         //0x14
    ComDataOverflow,                //0x15
    ComBadErase,                    //0x16
    ComBadProgram,                  //0x17
    InvalidTargetPressure,          //0x18
    SensorDisconnected,             //0x19
    SysUpperAlarm,                  //0x1a
    SysLowerAlarm,                  //0x1b
    DiaUpperAlarm,                  //0x1c
    DiaLowerAlarm,                  //0x1d
    MAPUpperAlarm,                  //0x1e
    MAPLowerAlarm,                  //0x1f
    Abort,                          //0x20
    NotAvail,                       //0x21
    Splat,                          //0x22
    Artifact,                       //0x23
    LowInflation,                   //0x24
    KinkedTube,                     //0x25
    EarlyDump,                      //0x26
    NotEnoughData,                  //0x27
    PatientInvalid,                 //0x28
    ReinflateTooLate,               //0x29
    ReinflateLimit,                 //0x2a
    Bleed,                          //0x2b
    Leak,                           //0x2c
    SafetyFault,                    //0x2d
    TubeMismatch,                   //0x2e
    NotFunctional,                  //0x2f
}

#[derive(Debug,Default,Clone)]
pub enum SPO2Error{
    #[default]
    None,                           //0x00
    BoardFail,                      //0x01
    BoardFailStandby,               //0x02
    BadProbe,                       //0x03
    WriteConfig,                    //0x04
    SPO2Only,                       //0x05
    InterferenceDetected,           //0x06
    LowPerfusionIndex,              //0x07
    SensorDisconnected,             //0x08
    SaturationUpperAlarm,           //0x09
    SaturationLowerAlarm,           //0x0a
    SearchingForBoard,              //0x0b
    NoProbe,                        //0x0c
    IncorrectProbe,                 //0x0d
    SearchingForPulse,              //0x0e
    ProbeExpired,                   //0x0f
    NoCable,                        //0x10
    IncorrectCable,                 //0x11
    BadCable,                       //0x12
    CableExpired,                   //0x13
    ValueNotAcquired,               //0x14
    LowConfidence,                  //0x15
    NotFunctional,                  //0x16
}


#[derive(Debug,Default,Clone)]
pub enum CO2Error{
    #[default]
    None,                           //0
    NoBreath,                       //1
    SensorDisconnected,             //2
    EndTidalUpperAlarm,             //3
    EndTidalLowerAlarm,             //4
    FractionInspiredUpperAlarm,     //5
    Reserved,                       //6
    IPILowerAlarm,                  //7
    NotFunctional,                  //8
}

#[derive(Debug,Default,Clone)]
pub enum HRPRError{
    #[default]
    None,                       //0
    SensorDisconnected,         //1
    HRPRUpperAlarm,             //2
    HRPRLowerAlarm,             //3
    DifficultyAcquiring,        //4
    ExtendedLowConfidence,      //5
}

#[derive(Debug,Default,Clone)]
pub enum SPHBError{
    #[default]
    None,                   //0
    WriteConfig,            //1
    LowPerfusionIndex,      //2
    SensorDisconnected,     //3
    SPHBUpperAlarm,         //4
    SPHBLowerAlarm,         //5
    LowConfidence,          //6
    NotFunctional,          //7
}

#[derive(Debug,Default,Clone)]
pub enum SPOCError{
    #[default]
    None,               //0
    WriteConfig,        //1
    LowProfusionIndex,  //2
}

#[derive(Debug,Default,Clone)]
pub enum SPCOError{
    #[default]
    None,               //0
    WriteConfig,        //1
    LowProfusionIndex,  //2
}

#[derive(Debug,Default,Clone)]
pub enum SPMETError{
    #[default]
    None,               //0
    WriteConfig,        //1
    LowProfusionIndex,  //2
}

#[derive(Debug,Default,Clone)]
pub enum RespirationError{
    #[default]
    None,                               //0x00
    WriteConfig,                        //0x01
    LowSignalStrength,                  //0x02
    RespiritoryPause,                   //0x03
    NoNewRRCalculation,                 //0x04
    SensorDisconnected,                 //0x05
    RespUpperAlarm,                     //0x06
    RespLowerAlarm,                     //0x07
    LowConfidence,                      //0x08
    NotFunctional,                      //0x09
    NoBreath,                           //0x0a
    InterferenceDetectedPatient,        //0x0b
    InterferenceDetectedBackground,     //0x0c
    DifficultyAcquiring,                //0x0d
    ExtendedLowConfidence,              //0x0e
}

#[derive(Debug,Default,Clone)]
pub enum TempError{
    #[default]
    None,                               //0x00
    ADPulseLong,                        //0x01
    ADPulseShort,                       //0x02
    ADRCalLong,                         //0x03
    ADRcalShort,                        //0x04
    ADPTBOver,                          //0x05
    ADPTBUnder,                         //0x06
    AD,                                 //0x07
    ADProbeCalib,                       //0x08
    ProbeWellRequired,                  //0x09
    ConfigTransferFailure,              //0x0a
    ConfigCRCFail,                      //0x0b
    ConfigCRCFailProbe,                 //0x0c
    ConfigCRCFailLog,                   //0x0d
    ConfigCalCRCFail,                   //0x0e
    ConfigNoProbe,                      //0x0f
    ConfigUninitProbe,                  //0x10
    EventsInvalidIndex,                 //0x11
    ConfigSlaveAckFailure,              //0x12
    ConfigNullCharFailure,              //0x13
    ConfigAcquireFaiure,                //0x14
    ConfigReleaseFailure,               //0x15
    ConfigInvalidPTRFailure,            //0x16
    ConfigInitFailure,                  //0x17
    HeaterOn,                           //0x18
    HeaterOff,                          //0x19
    HeaterVoltageHiighQ,                //0x1a
    HeaterVoltageQHighImpedance,        //0x1b
    HeaterVoltageLow,                   //0x1c
    HeaterFailsafe,                     //0x1d
    ProbeOverTemp,                      //0x1e
    ExcessiveHeaterEnergy,              //0x1f
    HostInterfaceError,                 //0x20
    OverAmbientTemp,                    //0x21
    UnderAmbientTemp,                   //0x22
    InvalidSuretempAlgorithm,           //0x23
    OverMaxBattVolts,                   //0x24
    UnderMinBattVolts,                  //0x25
    BattVoltsNotSet,                    //0x26
    PredictionAlgorithmNotSet,          //0x27
    AmbientTemperatureNotSet,           //0x28
    NoProbeTempRise,                    //0x29
    BadProbeGain,                       //0x2a
    BadProbeResponseValue,              //0x2b
    TimeLimitExceeded,                  //0x2c
    SensorDisconnected,                 //0x2d
    TempUpperAlarm,                     //0x2e
    TempLowerAlarm,                     //0x2f
    TissueContactLoss,                  //0x30
    ReplaceProbe,                       //0x31
    RetryMeasurement,                   //0x32
    NotFunctional,                      //0x33
}

#[derive(Debug,Default,Clone)]
pub enum DisplayError{
    #[default]
    None,                       //0
    DisplayBacklightFailure,    //1
}

#[derive(Debug,Default,Clone)]
pub enum BatteryError{
    #[default]
    None,                   //0
    BatteryMissing,         //1
    BatteryChargeFailure,   //2
    BatteryLow,             //3
    BatteryVeryLow,         //4
    BatteryMode,            //5
}

#[derive(Debug,Default,Clone)]
pub enum PrinterError{
    #[default]
    None,                   //0x00
    VoltageError,           //0x01
    RecieveDataError,       //0x02
    HeadUp,                 //0x03
    CutterError,            //0x04
    HardwareError,          //0x05
    ThermalError,           //0x06
    MotorError,             //0x07
    PaperNearEnd,           //0x08
    MarkDetectionError,     //0x09
    PaperOut,               //0x0a
    Jam,                    //0x0b
    OtherError,             //0x0c
}

#[derive(Debug,Default,Clone)]
pub enum ScaleError{
    #[default]
    None,           //0
    OverCapacity,   //1
    AnalogError,    //2
    NotFunctional,  //3
}

#[derive(Debug,Default,Clone)]
pub enum PatientMotionError{
    #[default]
    None,                               //0x00
    ExitingBed,                         //0x01
    OutOfBed,                           //0x02
    LowMotion,                          //0x03
    ExtremelyHighMotion,                //0x04
    AirMattressDetected,                //0x05
    TurnInterval,                       //0x06
    EnoughMotion,                       //0x07
    ProbeUpsideDown,                    //0x08
    ConfirmedPatientTurn,               //0x09
    UnconfirmedPatientTurn,             //0x0a
    ExitingChair,                       //0x0b
    OutOfChair,                         //0x0c
    BedSensorConnected,                 //0x0d
    BedSensorDisconnected,              //0x0e
    ChairSensorConnected,               //0x0f
    ChairSensorDisconnected,            //0x10
    LostSignal,                         //0x11
    UnstableSignal,                     //0x12
    BedSensorLifeNearExpiration,        //0x13
    BedSensorExpired,                   //0x14
    ChairSensorLifeNearExpiration,      //0x15
    ChairSensorExpired,                 //0x16
    BedSensorUpsideDown,                //0x17
    ChairSensorUpsideDown,              //0x18
    ModuleNotFunctional,                //0x19
    ConsiderUsingAirMattress,           //0x1a
    BadBedSignal,                       //0x1b
    DefectiveBedSensor,                 //0x1c
    BadChairSignal,                     //0x1d
    DefectiveChairSensor,               //0x1e
    BedSensorExpirationIn30Days,        //0x1f
    BedSensorExpirationIn7Days,         //0x20
    BedSensorExpirationIn1Day,          //0x21
    BedSensorExpirationIn4Hours,        //0x22
    BedSensorExpirationIn2Hours,        //0x23
    BedSensorExpirationIn1Hour,         //0x24
    ChairSensorExpirationIn30Days,      //0x25
    ChairSensorExpirationIn7Days,       //0x26
    ChairSensorExpirationIn1Day,        //0x27
    ChairSensorExpirationIn4Hours,      //0x28
    ChairSensorExpirationIn2Hours,      //0x29
    ChairSensorExpirationIn1Hour,       //0x2a
    DuplicateSensorConnected,           //0x2b
    BedChairConflict,                   //0x2c
}

#[derive(Debug,Default,Clone)]
pub enum SecurityProviderError{
    #[default]
    None,                                   //0x00
    NoProvidersConfiguredAtHost,            //0x01
    SecurityProviderInstanceNotFound,       //0x02
    SecurityProviderNotAvailable,           //0x03
    SecurityProviderInternalError,          //0x04
    SecurityProviderUnknownError,           //0x05
    UserNotFound,                           //0x06
    AuthFailInvalidUsernameOrPass,          //0x07
    AuthFailAccountDisabledOrExpired,       //0x08
    AuthFailPasswordExpired,                //0x09
    AuthFailGroupMembershipFail,            //0x0a
    AuthFailPasswordRequired,               //0x0b
    AuthFailPinRequired,                    //0x0c
}

#[derive(Debug,Default,Clone)]
pub enum UpdateHostEndError{
    #[default]
    None,                       //0
    PluginNotFound,             //1
    UpdateFileNotFound,         //2
    InvalidUpdateFile,          //3
    DeviceInventoryCorrupt,     //4
    DeviceLicenseCorrupt,       //5
    UnknownError,               //6
}

#[derive(Debug,Default,Clone)]
pub enum UpdateDeviceEndError{
    #[default]
    None,                       //0
    UnknownError,               //1
    FiileTransferTerminated,    //2
    FileCorrupted,              //3
    NotCompatible,              //4
    BadUpdateToken,             //5
    UnknownZipContents,         //6
    FileSystemOutOfSpace,       //7
}

#[derive(Debug,Default,Clone)]
pub enum ECGError{
    #[default]
    None,                       //0
    VentricularTachycardia,     //1
    VentricularFibrilation,     //2
    Asystole,                   //3
    NotFunctional,              //4
    UnableToAnalyze,            //5
    UnableToDetect,             //6
    ElectrodeOff,               //7
}

#[derive(Debug,Default,Clone)]
pub enum ErrSpecies{
    #[default]
    None,               //0x00
    WACPStack,          //0x01
    MSGProc,            //0x02
    AppInternal,        //0x03
    Reserved,           //0x04
    Common,             //0x05
    NIBP,               //0x06
    SPO2,               //0x07
    TYMP,               //0x08
    Printer,            //0x09
    Display,            //0x0a
    Battery,            //0x0b
    SPHB,               //0x0c
    SPOC,               //0x0d
    SPCO,               //0x0e
    SPMET,              //0x0f
    Respiration,        //0x10
    HRPR,               //0x11
    Scale,              //0x12
    PatientMotion,      //0x13
    SecurityProvider,   //0x14
    UpdateHostEnd,      //0x15
    UpdateDeviceEnd,    //0x16
}
impl ObjectSpecies for ErrSpecies{}

impl Object for Error{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Error }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Error }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ErrorSelect{
    error_class_id:u32,
    source:u16,
    error_code:&'static dyn ErrorCode,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ErrorSelect{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Error }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ErrorSpecies::ErrorSelect) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ExtendedError{
    error_code:&'static dyn Object,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ExtendedError{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Error }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::DataExt }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ErrorSpecies::ExtendedError) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ErrorDescription{
    io_param:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ErrorDescription{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Error }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ErrorSpecies::ErrorDescription) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

