use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug,Copy)]
pub enum FlashSpecies{
    FlashSegment,
    FlashCRC,
    DeviceFlashConfig,
    SectorFlashConfig,
    Mode,
    Description,
    PrepConfig,
    ProgConfig,
}
impl ObjectSpecies for FlashSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct FlashChunk{
    offset:u32,
    data:Vec<u8>,
    #[getter(skip)]
    raw_object:RawObject,
}


impl Object for FlashChunk{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Flash }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FlashSpecies::FlashCRC) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct FlashMsgArg{
    io_param_a:Result<ID, u32>,
    io_param_b:Option<u32>,
    #[getter(skip)]
    species:&'static FlashSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum ID{
    #[default]
    None,
    Bootloader,
    Application,
}

impl Object for FlashMsgArg{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Flash }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies>  { Box::new(*self.species) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
