use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use chrono;

#[derive(Clone,Debug,Copy)]
pub enum HeightSpecies{
    Data,
    Config,
}
impl ObjectSpecies for HeightSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct HeightData{
    date_time:chrono::NaiveDateTime,
    status:Status,
    height:u32,
    extended_status:ExtendedStatus,
    mode:Mode,
    method:Method,
    source:Source,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum Status{
    #[default]
    DataNotAvailable,
    DataComplete,
    DataFailure,
    DataException,
    DataProcessing,
}

#[derive(Debug,Default,Clone)]
pub enum ExtendedStatus{
    #[default]
    None,
}

#[derive(Debug,Default,Clone)]
pub enum Mode{
    #[default]
    None,
    Unknown,
}

#[derive(Debug,Default,Clone)]
pub enum Method{
    #[default]
    None,
    Unknown,
    Manual,
    Device,
    Remote,
}

#[derive(Debug,Default,Clone)]
pub enum Source{
    #[default]
    None,
    Unknown,
}

impl Object for HeightData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Height }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies>  { Box::new(HeightSpecies::Data) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct HeightConfig{
    enabled:bool,
    manual_height:bool,
    height_display_units:DisplayUnits,
    height_default:u32,
    external_port:Port,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum DisplayUnits{
    #[default]
    Centimetres,
    Inches,
    FeetInches,
}

#[derive(Debug,Default,Clone)]
pub enum Port{
    #[default]
    None,
    Unknown,
    Port1,
    Port2,
    Port3,
    Port4,
}

impl Object for HeightConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Height }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies>  { Box::new(HeightSpecies::Config) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
