use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug,Copy)]
pub enum HIMSpecies{
    Data,
    Response,
    Config,
    Select,
}
impl ObjectSpecies for HIMSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct HIMData{
    identifier:u16,
    sequence_number:u16,
    status:Status,
    subpriority:u8, //bits 0-7 of Rank u16
    origin:u8,//bits 8-9 of Rank u16
    priority:u8,//bits 10-14 of Rank u16
    alarm_active:bool,//bit 15 of Rank u16
    content:String,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Copy,Clone,Default)]
pub enum Status{
    #[default]
    None,
    Complete,
    Incomplete,
}

impl Object for HIMData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::HIM }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies>  { Box::new(HIMSpecies::Data) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct HIMResponse{
    identifier:u16,
    result:String,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for HIMResponse{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::HIM }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies>  { Box::new(HIMSpecies::Response) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct HIMConfig{
    enabled:bool,
    unicode_support:[u8;16],
    max_rows:u16,
    max_columns:u16,
    max_content_size:u16,
    max_simultaneous_messages:u8,
    supported_capabiliites:String,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for HIMConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::HIM }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies>  { Box::new(HIMSpecies::Config) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct HIMSelect{
    identifier:u16,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for HIMSelect{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::HIM }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies>  { Box::new(HIMSpecies::Select) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
