use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use chrono;
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum InterpSpecies{
    Data,
}
impl ObjectSpecies for InterpSpecies{}


#[derive(Getters,Builder,Clone)]
pub struct InterpData{
    modified_date:chrono::NaiveDateTime,
    edited_by:String,
    algorithm:Algorithm,
    reason_statements_enabled:bool,
    text:String,
    #[getter(skip)]
    raw_object:RawObject,
}


#[derive(Clone,Default,Debug)]
pub enum Algorithm{
    #[default]
    None,
    Means,
    User,
    Spirometry,
}
impl Object for InterpData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Interpretation }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(InterpSpecies::Data) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
