use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum ListSpecies{
    ExtendedData,
    Config,
}
impl ObjectSpecies for ListSpecies{}


#[derive(Getters,Builder,Clone)]
pub struct ListConfig{
    class_id:u32,
    total_num_elements:u16,
    element_byte_size:u32,
    list_packet_size:u16,
    accepted_num_elements:u16,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ListConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::List }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ListSpecies::Config) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ListData{
    objects:Vec<&'static dyn Object>,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ListData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::List }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::DataExt }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ListSpecies::ExtendedData) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
