use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum LocationSpecies{
    Config,
}
impl ObjectSpecies for LocationSpecies{}


#[derive(Getters,Builder,Clone)]
pub struct LocationData{
    facility:String,
    building:String,
    floor:String,
    unit:String,
    room:String,
    bed:String,
    source:Source,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Source{
    #[default]
    None,
    BoltedDown,
    HardWired,
    UserEntered,
}

impl Object for LocationData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::LocationID }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(LocationSpecies::Config) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
