use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum ModuleSpecies{
    StandardData,           //0x00
    Host,                   //0x01
    Hardware,               //0x02
    Bootloader,             //0x03
    BootRecovery,           //0x04
    InitialBootloader,      //0x05
    OperatingSystem,        //0x06
    Font,                   //0x07
    AuxHost,                //0x08
    AuxHardware,            //0x09
    AuxBootloader,          //0x0a
    AuxOperatingSystem,     //0x0b
    NIBPModule,             //0x10
    NIBPSafetyAlg,          //0x11
    NIBPStepAlg,            //0x12
    NIBPFastAlg,            //0x13
    NIBPNeonateStepAlg,     //0x14
    NIBPPoemAlg,            //0x15
    NIBPStepAOAlg,          //0x16
    TempModule,             //0x20
    SureTempAlg,            //0x21
    SPO2Module,             //0x28
    ECGModule,              //0x30
    Means,                  //0x31
    ECGAlg,                 //0x32
    Spirometry,             //0x38
    CO2Module,              //0x40
    Frontend,               //0xc0
    PPP,                    //0xc1
    SecureDigitalCard,      //0xc3
    Printer,                //0xc3
    CommsModule,            //0xc4
    Radio,                  //0xc5
    Barcode,                //0xc6
    Scale,                  //0xc7
    Battery,                //0xc8
    SPHBModule,             //0xc9
    SPMETModule,            //0xca
    SPOCModule,             //0xcb
    SPCOModule,             //0xcc
    RespirationModule,      //0xcd
    InterfaceDesignSpec,    //0xce
    PationMotionMmodule,    //0xcf
    CustomDataModule,       //0xd0
    MainMCU,                //0xd1
    AuxMCU,                 //0xd2
    Bluetooth,              //0xd3
    PRModule,               //0xd4
    All,                    //0xff
    ModuleConfig,           //0, param
}
#[derive(Clone,Debug)]
pub enum ModuleExtendedSpecies{
    Standard,
    Host,
    Sensor,
    All
}
impl ObjectSpecies for ModuleSpecies{}
impl ObjectSpecies for ModuleExtendedSpecies{}


#[derive(Getters,Builder,Clone)]
pub struct ModuleData{
    version:String,
    description:String,
    #[getter(skip)]
    species:ModuleSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ModuleData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Module }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ModuleConfig{
    enabled:bool,
    key:String,
    supports_continuous_data:bool,
    supports_spot_check_data:bool,
    supports_continuous_alarm:bool,
    supports_episodic_alarm:bool,
    compatibility_version:u8,
    #[getter(skip)]
    species:ModuleSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ModuleConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Module }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ModuleSelect{
    selector:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ModuleSelect{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Module }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ModuleSpecies::ModuleConfig) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ModuleExtendedData{
    module_container:Vec<ModuleData>,
    #[getter(skip)]
    species:ModuleExtendedSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ModuleExtendedData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Module }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
