use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum WaveSpecies{
    Standard,           //0
    ECGWave,            //1
    AudioWave,          //2
    LeadImpedanceWave,  //3
    SpiroWave,          //4
}
impl ObjectSpecies for WaveSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct WaveData{
    wave_identifier:WaveIdentifier,
    quality:u8,
    samples:Vec<i16>,
    #[getter(skip)]
    species:WaveSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum WaveIdentifier{    //From u16
    #[default]
    I,              //0x00
    II,             //0x01
    III,            //0x02
    AVR,            //0x03
    AVL,            //0x04
    AVF,            //0x05
    V1,             //0x06
    V2,             //0x07
    V3,             //0x08
    V4,             //0x09
    V5,             //0x0a
    V6,             //0x0b
    V7,             //0x0c
    V3R,            //0x0d
    V5R,            //0x0e
    X,              //0x0f
    Y,              //0x10
    Z,              //0x11
    V4R,            //0x12
    V6R,            //0x13
    V8,             //0x14
    V9,             //0x15
    SpiroFlow,      //0x20
    SpiroVolume,    //0x21
}

impl Object for WaveData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Wave }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct WaveConfig{
    is_active:bool,
    power_freq:Frequency,
    sample_frequency:i16,
    sample_size:u8,
    sample_count:i16,
    resolution:i32,
    #[getter(skip)]
    species:WaveSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum Frequency{ //From u16
    #[default]
    None,   //0
    Fifty,  //1
    Sixty,  //2
}

impl Object for WaveConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Wave }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
