use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

impl ObjectSpecies for WaveSampleAnnotateSpecies{}
impl ObjectSpecies for WaveSampleExtendedSpecies{}
impl ObjectSpecies for WaveSampleParamSpecies{}
impl ObjectSpecies for WaveSampleCommonSpecies{}
#[derive(Clone,Debug)]
pub enum WaveSampleParamSpecies{
    Select, //0x01
}

#[derive(Clone,Debug)]
pub enum WaveSampleAnnotateSpecies{
    Standard,   //0x00
}

#[derive(Clone,Debug)]
pub enum WaveSampleExtendedSpecies{
    Standard,   //0x00
    Config,     //0x01
    Data,       //0x02
}

#[derive(Clone,Debug,Copy)]
pub enum WaveSampleCommonSpecies{
    Standard,                   //0x00
    ECGWaveSample,              //0x01
    AudioWaveSample,            //0x02
    ECGImpedanceSample,         //0x03
    RWaveDetect,                //0x04
    PaceWaveDetection,          //0x05
    SpiroWaveSample,            //0x06
    I,                          //0x07
    II,                         //0x08
    III,                        //0x09
    AVL,                        //0x0a
    AVR,                        //0x0b
    AVF,                        //0x0c
    V1,                         //0x0d
    V2,                         //0x0e
    V3,                         //0x0f
    V4,                         //0x10
    V5,                         //0x11
    V6,                         //0x12
    V7,                         //0x13
    V3R,                        //0x14
    V5R,                        //0x15
    X,                          //0x16
    Y,                          //0x17
    Z,                          //0x18
    V4R,                        //0x19
    V6R,                        //0x1a
    V8,                         //0x1b
    V9,                         //0x1c
    SpiroFlow,                  //0x1d
    SpiroVolume,                //0x1e
    Pads,                       //0x1f
    Respiration,                //0x20
    Oscillimetric,              //0x21
    Manometer,                  //0x22
    SPO2Saturation,             //0x23
    SPO2Plethysmographic,       //0x24
    SIQ,                        //0x25
    CO2,                        //0x26
    IBP1,                       //0x27
    IBP2,                       //0x28
    IBP3,                       //0x29
    IBP4,                       //0x2a
    Temp1,                      //0x2b
    Temp2,                      //0x2c
    Audio,                      //0x2d
    SPO2PulseAmplitude,         //0x2e
    ECGElectrodeRL,             //0x2f
    ECGElectrodeLL,             //0x30
    ECGElectrodeRA,             //0x31
    ECGElectrodeLA,             //0x32
    ECGElectrodeV1,             //0x33
    ECGElectrodeV2,             //0x34
    ECGElectrodeV3,             //0x35
    ECGElectrodeV4,             //0x36
    ECGElectrodeV5,             //0x37
    ECGElectrodeV6,             //0x38
    All,                        //0x39
    RWaveDetectConfig,          //0x80  **LEGACY**
    PWaveDetectConfig,          //0x81  **LEGACY**
}


#[derive(Getters,Builder,Clone)]
pub struct StandardAnnotation{
    sequence_number:u32,
    sample_index:u16,
    r#annotation:Annotation,
    #[getter(skip)]
    species:&'static WaveSampleCommonSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum Annotation{
    #[default]
    None,               //0x00
    Pacer,              //0x01
    Qrs,                //0x02
    Sync,               //0x03
    DefibFired,         //0x04
    LeadOff,            //0x05
    BreathDetect,       //0x06
    EndBreathDetect,    //0x07
    PulseDetect,        //0x08
    PulseTone,          //0x09
    VFib,               //0x0a
    Asystole,           //0x0b
    VTach,              //0x0c
    PVCRun,             //0x0d
    VRhythm,            //0x0e
    Couplet,            //0x0f
    PVCMin,             //0x10
    Bigeminy,           //0x11
    Trigeminy,          //0x12
    Tachycardia,        //0x13
    Bradycardia,        //0x14
    Pause,              //0x15
    MissedBeat,         //0x16
    MultiFocal,         //0x17
    ROnT,               //0x18
    PacerNonCapture,    //0x19
    PacerNonFire,       //0x1a
    VSync,              //0x1b
    ApneaIndexTotal,    //0x1c
    Apnea10to19,        //0x1d
    Apnea20to29,        //0x1e
    Apnea30Plus,        //0x1f
    DesaturationCount,  //0x20
    LeadOn,             //0x21
    ElectrodeOn,        //0x22
    ElectrodeOff,       //0x23
}

impl Object for StandardAnnotation{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::WaveSample }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Annotate }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(*self.species) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct WaveSample{
    sequence_number:u32,
    sample:u8,
    quality:Quality,
    #[getter(skip)]
    species:WaveSampleCommonSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum Quality{
    #[default]
    None,
    Invalid,
    Valid,
    Dubious,
}

impl Object for WaveSample{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::WaveSample }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct WaveSampleConfig{
    is_active:bool,
    is_active_mut:bool,
    sample_frequency:i16,
    sample_frequency_mut:bool,
    sample_size:u8,
    sample_size_mut:bool,
    sample_count:i16,
    sample_count_mut:bool,
    channels:u8,
    channels_mut:bool,
    channels_sequence:u8,
    channels_sequence_mut:bool,
    annotation_enabled:bool,
    annotation_enabled_mut:bool,
    sample_frequency_exponent:i8,
    sample_frequency_exponent_mut:bool,
    center_point:u32,
    center_point_mut:bool,
    max:u32,
    max_mut:bool,
    r#units:Units,
    units_mut:bool,
    units_exponent:i8,
    units_exponent_mut:bool,
    resolution:u32,
    resolution_mut:bool,
    resolution_exponent:i8,
    resolution_exponent_mut:bool,
    sample_period:u32,
    sample_period_mut:bool,
    encoding:u8,
    encoding_mut:bool,
    mutable_permissions:bool,
    #[getter(skip)]
    species:WaveSampleCommonSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum Units{
    #[default]
    None,
    MetersOfMercury,
    Volts,
    Pascal,
    DegreesCelsius,
    DegreesFahrenheit,
    DegreesKelvin,
    Percent,
}

impl Object for WaveSampleConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::WaveSample }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct WaveSampleDetectConfig{
    is_active:bool,
    detect_channel:u8,
    #[getter(skip)]
    r_or_p:bool,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for WaveSampleDetectConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::WaveSample }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> {  
        match self.r_or_p{
            true => Box::new(WaveSampleCommonSpecies::RWaveDetectConfig),
            false => Box::new(WaveSampleCommonSpecies::PWaveDetectConfig),
        }
    }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct WaveSampleSelect{
    select:u8,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for WaveSampleSelect{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::WaveSample }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(WaveSampleParamSpecies::Select) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct WaveSampleExtended{
    container:Vec<WaveSample>,
    #[getter(skip)]
    species:WaveSampleExtendedSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}
impl Object for WaveSampleExtended{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::WaveSample }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
